// $Id$
/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.OrderedTermBase;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.taxeditor.editor.definedterm.operation.MoveDefinedTermOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author l.morris
 * @date 10 Jan 2012
 *
 */
public class DefinedTermDropAdapter extends ViewerDropAdapter {

	private final DefinedTermEditor editor;
	/**
	 * @param viewer
	 */
	protected DefinedTermDropAdapter(DefinedTermEditor editor) {
		super(editor.getViewer());
		this.editor = editor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerDropAdapter#performDrop(java.lang.Object)
	 */
	@Override
	public boolean performDrop(Object data) {

		TermBase target = (TermBase) getCurrentTarget();//can be vocab
		int currentLocation = getCurrentLocation();
		System.out.println("current location : " + currentLocation + " , target : " + target.getTitleCache());
		//DefinedTermBase[] droppedElements = (DefinedTermBase[]) data;
		Object[] droppedTerms = (Object[]) data;
		Collection<DefinedTermBase> sourceTerms = new ArrayList<DefinedTermBase>(); //Arrays.asList(droppedElements)

		for (Object droppedTerm : droppedTerms) {

			DefinedTermBase term = (DefinedTermBase) droppedTerm;
			sourceTerms.add(term);

		}

		AbstractPostOperation operation = new MoveDefinedTermOperation("Move Descriptions",
		        StoreUtil.getUndoContext(),
		        target,
		        sourceTerms,
		        editor,
		        currentLocation);
		//TODO: implement execute
		StoreUtil.executeOperation(operation);
		// select the newly moved objects
		editor.getViewer().setSelection(new StructuredSelection(sourceTerms.toArray(new TermBase[sourceTerms.size()])));

		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerDropAdapter#validateDrop(java.lang.Object, int, org.eclipse.swt.dnd.TransferData)
	 */
	@Override
	public boolean validateDrop(Object target, int operation,
			TransferData transferType) {
		boolean transferDataIsSupported = TermTransfer.getInstance().isSupportedType(transferType);
		// maybe don't need this - they will be all TermBase anyway
		return target instanceof TermBase && transferDataIsSupported;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerDropAdapter#dragOver(org.eclipse.swt.dnd.DropTargetEvent)
	 */
	@Override
	public void dragOver(DropTargetEvent event) {
	    super.dragOver(event);
	    TermBase target = (TermBase) getCurrentTarget();
	    if(!(target instanceof OrderedTermBase)) {
	        // this is to only disable insert between items
	        event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL;
	    }


	}

}
