// $Id$
/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm.input;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.taxeditor.store.CdmStore;



/**
 * @author n.hoffmann
 * @date Jan 24, 2012
 *
 */
public class TermEditorInput extends AbstractDefinedTermEditorInput<DefinedTerm> {

	private TermType termType;
	private Set<TermVocabulary<DefinedTermBase>> vocabularies;
	private List<String> termClasses = Arrays.asList(new String[]{
			DefinedTerm.class.getName()
	});
	
	public TermEditorInput(TermType termType) {
		this.termType = termType;
		vocabularies = new HashSet<TermVocabulary<DefinedTermBase>>();
		initialiseVocabularies();
	}
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	@Override
	public String getName() {
		return termType.getMessage();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.definedterm.input.AbstractDefinedTermEditorInput#getTermClasses()
	 */
	@Override
	public List<String> getTermClasses() {
		return termClasses;
	}
	
	public TermType getTermType() {
		return termType;
	}
	
	public void initialiseVocabularies() {
		if(vocabularies != null) {
			vocabularies.clear();
		} 
		List<TermVocabulary<DefinedTermBase>> vocs = CdmStore.getService(IVocabularyService.class).findByTermType(termType);
		vocabularies.addAll(vocs);			
	}
	
	public Set<TermVocabulary<DefinedTermBase>> getVocabularies() {
		return vocabularies;
	}

}