// $Id$
/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.taxeditor.editor.definedterm.input.TermEditorInput;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostTaxonOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author l.morris
 * @date 21 Dec 2011
 *
 */
public class CreateDefinedTermOperation extends AbstractPostTaxonOperation {

	

	private TermBase parentTermBase;
	private TermEditorInput definedTermInput;

	/**
	 * @param label
	 * @param undoContext
	 * @param postOperationEnabled
	 */
	public CreateDefinedTermOperation(String label,
			IUndoContext undoContext, 
			TermBase termBase,
			TermEditorInput definedTermInput,
			IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, postOperationEnabled);
		this.parentTermBase = termBase;
		this.definedTermInput = definedTermInput;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		DefinedTermBase newTerm = definedTermInput.getTermType().getEmptyDefinedTermBase();
		if (newTerm == null) {
			IStatus status = 
					new Status(IStatus.CANCEL, 
							StoreUtil.getPluginId(), 
							"Creation of term corresponding to type '" + definedTermInput.getTermType().getMessage() + "' is not yet supported");
			MessagingUtils.warningDialog("Cannot create term", newTerm, status);
			return status;
		}
		
		if (parentTermBase instanceof TermVocabulary){
			TermVocabulary vocabulary = (TermVocabulary) parentTermBase;
			vocabulary.addTerm(newTerm);
		} else if (parentTermBase instanceof DefinedTermBase) {
			DefinedTermBase parent = (DefinedTermBase) parentTermBase;			
			parent.addIncludes(newTerm);			
			TermVocabulary vocabulary = parent.getVocabulary();
			vocabulary.addTerm(newTerm);
		}
		
		return postExecute(newTerm);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

}
