// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.wizard;

import java.io.File;
import java.net.URI;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.ResourceManager;

import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.taxeditor.ui.dialog.selection.SelectionDialogFactory;

/**
 * <p>ImportFromFileDataSourceWizardPage class.</p>
 *
 * @author n.hoffmann
 * @created 04.08.2009
 * @version 1.0
 */
public class ClassificationChooserWizardPage extends WizardPage implements Listener{

	public static final String PAGE_NAME = "ClassificationChooserWizardPage";

	//classification
    private Text textClassification;
    private Classification classification;
    private Button btnBrowseClassification;

    private Button btnClear;

    //report
    private FileDialog fileDialogReport;
    private Text textFileReport;

	/**
	 * <p>Constructor for ImportFromFileDataSourceWizardPage.</p>
	 *
	 * @param title a {@link java.lang.String} object.
	 * @param description a {@link java.lang.String} object.
	 * @param extensions an array of {@link java.lang.String} objects.
	 */
	protected ClassificationChooserWizardPage(String title, String description) {
		super(PAGE_NAME);

		setTitle(title);

		setDescription(description);

	}

	/**
	 * <p>XML</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.io.wizard.ClassificationChooserWizardPage} object.
	 */
	protected static ClassificationChooserWizardPage createPage(){
		return new ClassificationChooserWizardPage("Configure import destinations", "Note: Selecting no classification will create a default one.");
	}



	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	/** {@inheritDoc} */
	@Override
    public void createControl(Composite parent) {
		final Composite composite = new Composite(parent, SWT.NULL);

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		composite.setLayout(gridLayout);

		//classification
		Label label = new Label(composite, SWT.NONE);
		label.setText("Classification");
		textClassification = new Text(composite, SWT.NONE);
		textClassification.setEnabled(false);
		textClassification.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		btnBrowseClassification = new Button(composite, SWT.NONE);
		btnBrowseClassification.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/open.gif"));
		btnBrowseClassification.addListener(SWT.Selection, this);
		btnClear = new Button(composite, SWT.NONE);
		btnClear.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/trash.gif"));
		btnClear.addListener(SWT.Selection, this);

		//report
		Label labelReportFile = new Label(composite, SWT.NONE);
        labelReportFile.setText("Report File");

        fileDialogReport = new FileDialog(parent.getShell());

        fileDialogReport.setFilterExtensions(new String[]{"*.*"});

        textFileReport = new Text(composite, SWT.BORDER);
        textFileReport.setEditable(false);
        textFileReport.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));


        Button buttonReport = new Button(composite, SWT.PUSH);
        buttonReport.setText("Browse...");

        buttonReport.addSelectionListener(new SelectionAdapter(){

            /* (non-Javadoc)
             * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            @Override
            public void widgetSelected(SelectionEvent e) {
                String path = fileDialogReport.open();
                if(path!=null){
                    textFileReport.setText(path);
                    setPageComplete(true);
                }
            }

        });

		setControl(composite);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
	 */
	@Override
	public void handleEvent(Event event) {
	    if(event.widget==btnBrowseClassification){
	        classification = SelectionDialogFactory.getSelectionFromDialog(Classification.class, getShell(), null, null);
	        if(classification!=null){
	            textClassification.setText(classification.getTitleCache());
	        }
	    }
	    else if(event.widget==btnClear){
	        classification = null;
	        textClassification.setText("");
	    }
	}

    /**
     * @return the classification
     */
    public Classification getClassification() {
        return classification;
    }

    public URI getReportUri(){
        String text = textFileReport.getText();
        if(text==null || text.isEmpty()){
            return null;
        }
        return new File(text).toURI();
    }
}
