// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;

/**
 * Extending <code>TextSelection</code> allows us to keep using Eclipse's text selection, while
 * <code>IStructuredSelection</code> can be used to send an object associated with the selection,
 * i.e. a property source, to the workbench selection service.
 *
 * @author p.ciardelli
 * @created 03.07.2009
 * @version 1.0
 */
public class LineSelection extends TextSelection implements IStructuredSelection {

	private StructuredSelection structuredSelection;

	/**
	 * <p>Constructor for LineSelection.</p>
	 *
	 * @param selection a {@link org.eclipse.jface.text.ITextSelection} object.
	 * @param document a {@link org.eclipse.jface.text.IDocument} object.
	 * @param selectedObject a {@link java.lang.Object} object.
	 */
	public LineSelection(ITextSelection selection, IDocument document, Object selectedObject) {
		super(document, selection.getOffset(), selection.getLength());
		if (selectedObject != null) {
			this.structuredSelection = new StructuredSelection(selectedObject);
		} else {
			this.structuredSelection = StructuredSelection.EMPTY;
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#getFirstElement()
	 */
	/**
	 * <p>getFirstElement</p>
	 *
	 * @return a {@link java.lang.Object} object.
	 */
	@Override
    public Object getFirstElement() {
		return structuredSelection.getFirstElement();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#iterator()
	 */
	/**
	 * <p>iterator</p>
	 *
	 * @return a {@link java.util.Iterator} object.
	 */
	@Override
    public Iterator<?> iterator() {
		return structuredSelection.iterator();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#size()
	 */
	/**
	 * <p>size</p>
	 *
	 * @return a int.
	 */
	@Override
    public int size() {
		return structuredSelection.size();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#toArray()
	 */
	/**
	 * <p>toArray</p>
	 *
	 * @return an array of {@link java.lang.Object} objects.
	 */
	@Override
    public Object[] toArray() {
		return structuredSelection.toArray();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#toList()
	 */
	/**
	 * <p>toList</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	@Override
    public List<?> toList() {
		return structuredSelection.toList();
	}
}
