// $Id$
/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.operation;

import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.ICdmBase;

/**
 * @author pplitzner
 * @date 03.12.2013
 *
 */
public abstract class AbstractPostOperation<T extends ICdmBase> extends AbstractOperation {


    /**
     * A reference to the {@link ICdmBase} element the concrete operation is working on
     */
    protected T element;

    /**
     *
     */
    protected IPostOperationEnabled postOperationEnabled;


    /**
     * <p>Constructor for AbstractPostOperation.</p>
     *
     * @param label a {@link java.lang.String} object.
     * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
     */
    protected AbstractPostOperation(String label, IUndoContext undoContext) {
        super(label);
        addContext(undoContext);
    }

    /**
     * <p>Constructor for AbstractPostOperation.</p>
     *
     * @param label a {@link java.lang.String} object.
     * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
     * @param element a {@link ICdmBase} object on which this operation is working.
     */
    public AbstractPostOperation(String label, IUndoContext undoContext, T element) {
        this(label, undoContext);
        this.element = element;
    }

    /**
     * @param label
     * @param element
     * @param postOperationEnabled
     */
    public AbstractPostOperation(String label, IUndoContext undoContext,
            T element, IPostOperationEnabled postOperationEnabled) {
        this(label, undoContext);
        this.element = element;
        this.postOperationEnabled = postOperationEnabled;
    }

    /**
     * This method will try to call the post operation on a possibly registered
     * IPostOperationEnabled implementor. Objects that were affected by the operation
     * may be passed to the registered IPostOperationEnabled implementor.
     *
     * @param objectAffectedByOperation the affected object. Should be <code>null</code> if not needed
     * @return a {@link org.eclipse.core.runtime.IStatus} object.
     */
    protected IStatus postExecute(CdmBase objectAffectedByOperation) {
    	if(postOperationEnabled != null){
    		return postOperationEnabled.postOperation(objectAffectedByOperation) ? Status.OK_STATUS : Status.CANCEL_STATUS;
    	}
    	return Status.OK_STATUS;
    }

    /**
     * <p>Getter for the field <code>postOperationEnabled</code>.</p>
     *
     * @return a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
     */
    public IPostOperationEnabled getPostOperationEnabled() {
    	return postOperationEnabled;
    }

}