// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Dec 6, 2010
 * @version 1.0
 */
public class FeatureSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<Feature> {

	public static Feature select(Shell shell, ConversationHolder conversation, Feature feature){
		FeatureSelectionDialog dialog = new FeatureSelectionDialog(shell, conversation,
				"Choose Feature", false, FeatureSelectionDialog.class.getCanonicalName(), feature);
		return getSelectionFromDialog(dialog);
	}
	
	/**
	 * @param shell
	 * @param conversation
	 * @param title
	 * @param multi
	 * @param settings
	 * @param cdmObject
	 */
	protected FeatureSelectionDialog(Shell shell,
			ConversationHolder conversation, String title, boolean multi,
			String settings, Feature cdmObject) {
		super(shell, conversation, title, multi, settings, cdmObject);
	}

	@Override
	protected Feature getPersistentObject(UUID uuid) {
		
		DefinedTermBase<?> term = CdmStore.getService(ITermService.class).load(uuid);
		
		if(term instanceof Feature){
			return (Feature) term;
		}
		
		return null;
	}

	@Override
	protected void initModel() {
		List<Feature> features = CdmStore.getService(ITermService.class).list(Feature.class, null, null, null, null);
		
		List<UuidAndTitleCache<Feature>> featureUuidAndTitleCache = new ArrayList<UuidAndTitleCache<Feature>>();
		
		for(Feature feature : features){
			UuidAndTitleCache<Feature> uuidAndTitleCache = new UuidAndTitleCache<Feature>(Feature.class, feature.getUuid(), feature.getId(), feature.getTitleCache());
			featureUuidAndTitleCache.add(uuidAndTitleCache);
		}
		
		model =  featureUuidAndTitleCache;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewWizardLinkText()
	 */
	@Override
	protected String getNewWizardLinkText() {
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewEntityWizard()
	 */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return null;
	}

}
