// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;
import org.springframework.security.core.GrantedAuthority;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IGrantedAuthorityService;
import eu.etaxonomy.cdm.model.common.GrantedAuthorityImpl;
import eu.etaxonomy.cdm.persistence.dto.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewGrantedAuthorityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityLabelTextProvider;

/**
 * @author a.kohlbecker
 * @created Sept 12, 2011
 * @version 1.0
 */
public class GrantedAuthoritySelectionDialog extends	AbstractFilteredCdmResourceSelectionDialog<GrantedAuthorityImpl> {

	public static GrantedAuthority select(Shell shell, ConversationHolder conversation, GrantedAuthorityImpl authority){

		GrantedAuthoritySelectionDialog dialog = new GrantedAuthoritySelectionDialog(shell, conversation,
				"Choose Granted Authority", false, GrantedAuthoritySelectionDialog.class.getCanonicalName(), authority);

		return getSelectionFromDialog(dialog);
	}

	/**
	 * @param shell
	 * @param conversation
	 * @param title
	 * @param multi
	 * @param settings
	 * @param cdmObject
	 */
	protected GrantedAuthoritySelectionDialog(Shell shell,
			ConversationHolder conversation, String title, boolean multi,
			String settings, GrantedAuthorityImpl cdmObject) {
		super(shell, conversation, title, multi, settings, cdmObject);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {
		List<GrantedAuthorityImpl> authorities = CdmStore.getService(IGrantedAuthorityService.class).list(GrantedAuthorityImpl.class, null, null, null, null);

		model = new ArrayList<UuidAndTitleCache<GrantedAuthorityImpl>>();

		for(GrantedAuthorityImpl authority : authorities){
			
			model.add(new UuidAndTitleCache<GrantedAuthorityImpl>(GrantedAuthorityImpl.class, authority.getUuid(), authority.getId(), String.format("%s", GrantedAuthorityLabelTextProvider.getText(authority))));
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewWizardLinkText()
	 */
	@Override
	protected String getNewWizardLinkText() {
		return String.format("Create a new <a>%1s</a>", "GrantedAuthority ");
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return new NewGrantedAuthorityWizard();
	}

	@Override
	protected GrantedAuthorityImpl getPersistentObject(UUID uuid) {
		return CdmStore.getService(IGrantedAuthorityService.class).load(uuid);
	}

	@Override
	protected String getTitle(GrantedAuthorityImpl authority) {
		return authority != null ? GrantedAuthorityLabelTextProvider.getText(authority) : "";
	}

}
