// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.element;

import java.util.ArrayList;
import java.util.List;




/**
 * <p>Abstract AbstractCdmFormElementWithErrors class.</p>
 *
 * @author n.hoffmann
 * @created Oct 26, 2010
 * @version 1.0
 */
public abstract class AbstractCdmFormElementWithErrors extends AbstractCdmFormElement implements IErrorIntolerableElement {

	private List<IExceptionHandler> exceptionHandlers = new ArrayList<IExceptionHandler>();

	/**
	 * <p>Constructor for AbstractCdmFormElementWithErrors.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 */
	public AbstractCdmFormElementWithErrors(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IErrorIntolerableElement#addExceptionHandler(eu.etaxonomy.taxeditor.forms.IExceptionHandler)
	 */
	/** {@inheritDoc} */
	@Override
	public void addExceptionHandler(IExceptionHandler handler) {
		exceptionHandlers.add(handler);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IErrorIntolerableElement#removeExceptionHandler(eu.etaxonomy.taxeditor.forms.IExceptionHandler)
	 */
	/** {@inheritDoc} */
	@Override
	public void removeExceptionHandler(IExceptionHandler handler) {
		exceptionHandlers.remove(handler);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IErrorIntolerableElement#exceptionOccurred()
	 */
	/** {@inheritDoc} */
	@Override
	public void exceptionOccurred(CdmPropertyChangeEvent event) {
		for(IExceptionHandler handler : exceptionHandlers){
			handler.handleException(event);
		}
	}

}
