// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * <p>CheckboxElement class.</p>
 *
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class CheckboxElement extends AbstractCdmFormElement implements SelectionListener, IEnableableFormElement, ISelectable {

	private final Button checkbox;

	private Label label;

	/**
	 * <p>Constructor for CheckboxElement.</p>
	 *
	 * @param initialState a boolean.
	 * @param style a int.
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param labelString a {@link java.lang.String} object.
	 */
	protected CheckboxElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString, boolean initialState,
			int style) {
		super(formFactory, parentElement);

		if(labelString != null){
			label = formFactory.createLabel(getLayoutComposite(), labelString);
			addControl(label);
		}

		checkbox = formFactory.createButton(getLayoutComposite(), null, SWT.CHECK | style);
		checkbox.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		checkbox.setSelection(initialState);

		checkbox.addSelectionListener(this);
		addControl(checkbox);
	}

	/**
	 * <p>setSelection</p>
	 *
	 * @param selected a boolean.
	 */
	public void setSelection(boolean selected){
		checkbox.removeSelectionListener(this);
		checkbox.setSelection(selected);
		checkbox.addSelectionListener(this);
	}

	/**
	 * <p>getSelection</p>
	 *
	 * @return a boolean.
	 */
	public boolean getSelection(){
		return checkbox.getSelection();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	/** {@inheritDoc} */
	@Override
    public void widgetSelected(SelectionEvent e) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}

	/** {@inheritDoc} */
	@Override
    public void widgetDefaultSelected(SelectionEvent e) {}

	/** {@inheritDoc} */
	@Override
    public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	/** {@inheritDoc} */
	@Override
    public void setEnabled(boolean enabled) {
		checkbox.setEnabled(enabled);
		label.setEnabled(enabled);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement#isEnabled()
	 */
	@Override
	public boolean isEnabled() {
	    return checkbox.isEnabled();
	}

	/** {@inheritDoc} */
	@Override
    public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_COMPOSITE_BACKGROUND;

		Color color = getColor(colorId);
		setBackground(color);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement#setBackground(org.eclipse.swt.graphics.Color)
	 */
	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		checkbox.setBackground(color);
		if (label != null) {
            label.setBackground(color);
        }
	}
}
