// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.element;

import java.net.MalformedURLException;
import java.net.URI;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.taxeditor.Messages;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author n.hoffmann
 * @created Dec 20, 2010
 * @version 1.0
 */
public class UriWithLabelElement extends TextWithLabelElement {

    private final Label labelException;
    private final Button btnOpenBrowser;

	protected UriWithLabelElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, String labelString,
			URI initialUri, Integer textHeight, int style) {
		super(formFactory, parentElement, false);

		//label
        initLabel(formFactory, labelString, false, getLayoutComposite());

        //composite(uri + button)
        Composite textAndButton = formFactory.createComposite(getLayoutComposite(), style);
        addControl(textAndButton);
        textAndButton.setLayout(LayoutConstants.LAYOUT(2, false));
        textAndButton.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

        //uri text
        initText(formFactory, null, textHeight, null, false, style, textAndButton);

        //button
		btnOpenBrowser = formFactory.createButton(textAndButton, "", SWT.NONE); //$NON-NLS-1$
		btnOpenBrowser.setImage(ImageResources.getImage(ImageResources.WEB));
		btnOpenBrowser.setToolTipText(Messages.UriWithLabelElement_OPEN_EXTERNAL_BROWSER);
		btnOpenBrowser.addSelectionListener(new SelectionAdapter() {
		    @Override
		    public void widgetSelected(SelectionEvent e) {
		        String errorTitle = Messages.UriWithLabelElement_INVALID_URL;
		        String errorText = Messages.UriWithLabelElement_COULD_NOT_OPEN_BROWSER;

		        URI uri = getUri();
		        if(uri!=null){
		            try {
                        PlatformUI.getWorkbench().getBrowserSupport().getExternalBrowser().openURL(uri.toURL());
                    } catch (PartInitException pie) {
                        MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, pie));
                    } catch (MalformedURLException mue) {
                        MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, mue));
                    } catch (IllegalArgumentException iae) {
                        MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, iae));
                    }
		        }
		        else{
		            MessagingUtils.informationDialog(errorTitle, new Status(IStatus.WARNING, TaxeditorStorePlugin.PLUGIN_ID, errorText, null));
		        }
		    }
        });
		btnOpenBrowser.setLayoutData(LayoutConstants.RIGHT());

		labelException = formFactory.createLabel(getLayoutComposite(), "", SWT.WRAP); //$NON-NLS-1$
		int numColumns = AbstractFormSection.DEFAULT_NUM_COLUMNS;
		if(getLayoutComposite().getLayout() instanceof TableWrapLayout){
		    numColumns = ((TableWrapLayout)getLayoutComposite().getLayout()).numColumns;
		}
        labelException.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(numColumns, 1));
		addControl(labelException);
		setUri(initialUri);
	}

	public void setUri(URI uri) {
		if(uri != null){
			super.setText(uri.toString());
		}
	}

	public URI getUri(){
        try {
            labelException.setFont(JFaceResources.getFontRegistry().get(JFaceResources.DEFAULT_FONT));
            labelException.setForeground(getPersistentBackground());
            labelException.setText(""); //$NON-NLS-1$
            return new URI(super.getText());
        } catch (Exception e) {
            labelException.setFont(JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT));
            labelException.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
            labelException.setText(Messages.UriWithLabelElement_URL_NOT_SAVED+e.getMessage());
            return null;
        }
	}

}
