/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.group.grantedauthority;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.wb.swt.SWTResourceManager;

/**
 * Table header widget for {@link CdmAuthorityComposite}
 * 
 * @author cmathew
 * @created Mar 28, 2013
 *
 */
public class CdmAuthorityTableHeader extends Composite {

	private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
	private CLabel lblUuid;
	private CLabel lblOperation;
	/**
	 * Create the composite, with 5 header fields.
	 * 
	 * @param parent
	 * @param style
	 */
	public CdmAuthorityTableHeader(Composite parent, int style) {
		super(parent, SWT.NONE);
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				toolkit.dispose();
			}
		});
		toolkit.adapt(this);
		toolkit.paintBordersFor(this);
		setLayout(new GridLayout(5, false));

		Label lblDirtyFlag = new Label(this, SWT.NONE);
		GridData gd_lblDirtyFlag = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_lblDirtyFlag.widthHint = 15;
		lblDirtyFlag.setLayoutData(gd_lblDirtyFlag);

		toolkit.adapt(lblDirtyFlag, true, true);
		
		CLabel lblType = new CLabel(this, SWT.NONE);
		GridData gd_lblType = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_lblType.widthHint = 200;
		lblType.setLayoutData(gd_lblType);
		lblType.setText("TYPE");
		lblType.setForeground(SWTResourceManager.getColor(SWT.COLOR_BLUE));
		lblType.setAlignment(SWT.CENTER);
		toolkit.adapt(lblType);
		toolkit.paintBordersFor(lblType);

		CLabel lblEntity = new CLabel(this, SWT.NONE);
		lblEntity.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLUE));
		lblEntity.setAlignment(SWT.CENTER);
		GridData gd_lblEntity = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_lblEntity.widthHint = 200;
		lblEntity.setLayoutData(gd_lblEntity);
		toolkit.adapt(lblEntity);
		toolkit.paintBordersFor(lblEntity);
		lblEntity.setText("ENTITY");

		//createUUIDHeader();
		
		lblOperation = new CLabel(this, SWT.NONE);
		lblOperation.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLUE));
		lblOperation.setAlignment(SWT.CENTER);
		GridData gd_lblOperation = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_lblOperation.widthHint = 150;
		lblOperation.setLayoutData(gd_lblOperation);
		toolkit.adapt(lblOperation);
		toolkit.paintBordersFor(lblOperation);
		lblOperation.setText("OPERATION");

	}


	public void setUuidHeaderVisible(boolean visible) {
		if(visible) {
			if(lblUuid == null) {
				createUUIDHeader();
			}
			lblUuid.moveAbove(lblOperation);			
		} else {
			if(lblUuid != null) {
				lblUuid.dispose();
			}
			lblUuid = null;
		}
		layout();
	}
	
	private void createUUIDHeader() {
		lblUuid = new CLabel(this, SWT.NONE);
		lblUuid.setForeground(SWTResourceManager.getColor(SWT.COLOR_BLUE));
		lblUuid.setAlignment(SWT.CENTER);
		GridData gd_lblUuid = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_lblUuid.widthHint = 270;
		lblUuid.setLayoutData(gd_lblUuid);
		toolkit.adapt(lblUuid);
		toolkit.paintBordersFor(lblUuid);
		lblUuid.setText("UUID");
	}

}
