// $Id$
/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.mvc;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.mvc.combo.EnumTermCombo;
import eu.etaxonomy.taxeditor.ui.mvc.element.TextWithLabelElementMVC;

/**
 * @author pplitzner
 * @date 14.04.2014
 *
 */
public class DerivedUnitGeneralDetailElementMVC extends AbstractCdmComposite {

    private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
    private final TextWithLabelElementMVC textWithLabelElement;
    private final EnumTermCombo enumTermCombo;

    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public DerivedUnitGeneralDetailElementMVC(Composite parent, int style) {
        super(parent, style);
        addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent e) {
                toolkit.dispose();
            }
        });
        toolkit.adapt(this);
        toolkit.paintBordersFor(this);
        {
            TableWrapLayout tableWrapLayout = new TableWrapLayout();
            tableWrapLayout.numColumns = 2;
            setLayout(tableWrapLayout);
        }

                Label lblTitleCache = toolkit.createLabel(this, "Title Cache", SWT.NONE);
                new Label(this, SWT.NONE);

                Label lblSpecorobstype = toolkit.createLabel(this, "Spec./Obs. Type", SWT.NONE);

                enumTermCombo = new EnumTermCombo(this, SWT.NONE, SpecimenOrObservationType.class);
                TableWrapData twd_enumTermCombo = new TableWrapData(TableWrapData.LEFT, TableWrapData.TOP, 1, 1);
                twd_enumTermCombo.grabHorizontal = true;
                enumTermCombo.setLayoutData(twd_enumTermCombo);
                toolkit.adapt(enumTermCombo);
                toolkit.paintBordersFor(enumTermCombo);

        Label lblCountry = toolkit.createLabel(this, "Country", SWT.NONE);
        new Label(this, SWT.NONE);

        Label lblLocality = toolkit.createLabel(this, "Locality", SWT.NONE);
        new Label(this, SWT.NONE);

        GeographicPointElement geographicPointElement = new GeographicPointElement(this, SWT.NONE);
        geographicPointElement.setLayoutData(new TableWrapData(TableWrapData.LEFT, TableWrapData.TOP, 1, 2));
        toolkit.adapt(geographicPointElement);
        toolkit.paintBordersFor(geographicPointElement);

        Label lblElevation = toolkit.createLabel(this, "Elevation", SWT.NONE);
        new Label(this, SWT.NONE);

        Label lblDate = toolkit.createLabel(this, "Date", SWT.NONE);
        new Label(this, SWT.NONE);

        Label lblCollector = toolkit.createLabel(this, "Collector", SWT.NONE);
        new Label(this, SWT.NONE);

        Label lblCollectingNumber = toolkit.createLabel(this, "Collecting Number", SWT.NONE);

        TextWithLabelElementMVC textWithLabelElement_1 = new TextWithLabelElementMVC(this, (String) null, (Integer) null, 0);
        toolkit.adapt(textWithLabelElement_1);
        toolkit.paintBordersFor(textWithLabelElement_1);

        Label lblCollection = toolkit.createLabel(this, "Collection", SWT.NONE);
        new Label(this, SWT.NONE);

        Label lblAccessionNumber = toolkit.createLabel(this, "Accession Number", SWT.NONE);

        textWithLabelElement = new TextWithLabelElementMVC(this, (String) null, (Integer) null, 0);
        TableWrapLayout tableWrapLayout = (TableWrapLayout) textWithLabelElement.getLayout();
        tableWrapLayout.verticalSpacing = 0;
        tableWrapLayout.topMargin = 0;
        tableWrapLayout.horizontalSpacing = 0;
        tableWrapLayout.bottomMargin = 0;
        TableWrapData twd_textWithLabelElement = new TableWrapData(TableWrapData.LEFT, TableWrapData.TOP, 1, 1);
        twd_textWithLabelElement.align = TableWrapData.FILL;
        twd_textWithLabelElement.grabHorizontal = true;
        textWithLabelElement.setLayoutData(twd_textWithLabelElement);
        toolkit.adapt(textWithLabelElement);
        toolkit.paintBordersFor(textWithLabelElement);

    }

    public TextWithLabelElementMVC getTextWithLabelElement() {
        return textWithLabelElement;
    }
    public EnumTermCombo getEnumTermCombo() {
        return enumTermCombo;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.mvc.interfaces.CdmComposite#initController(java.lang.Object, eu.etaxonomy.taxeditor.ui.element.CdmFormFactory, eu.etaxonomy.taxeditor.ui.element.ICdmFormElement)
     */
    @Override
    public void initInternalController(CdmFormFactory formFactory, ICdmFormElement parentElement) {
        // empty on purpose
    }
}
