// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.Collection;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * <p>DescriptionElementMediaSection class.</p>
 *
 * @author n.hoffmann
 * @created Mar 18, 2010
 * @version 1.0
 */
public class DescriptionElementMediaSection extends
		AbstractEntityCollectionSection<DescriptionElementBase, Media> {

	public DescriptionElementMediaSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Media", style);
	}

	/** {@inheritDoc} */
	@Override
	public void setEntity(DescriptionElementBase entity) {
		if(entity.getFeature().equals(Feature.IMAGE())){
			this.setVisible(false);
		}
		super.setEntity(entity);
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(Media element) {
		getEntity().addMedia(element);
	}

	/** {@inheritDoc} */
	@Override
	public Media createNewElement() {
		return Media.NewInstance();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Media> getCollection(DescriptionElementBase entity) {
		return entity.getMedia();
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No media yet.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Create new media";
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(Media element) {
		getEntity().removeMedia(element);
	}


}
