// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.Collection;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * <p>GeoScopeSection class.</p>
 *
 * @author n.hoffmann
 * @created Sep 17, 2010
 * @version 1.0
 */
public class GeoScopeSection extends AbstractEntityCollectionSection<TaxonDescription, NamedArea> {
	
	/**
	 * <p>Constructor for GeoScopeSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public GeoScopeSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(formFactory, conversation, parentElement, "Geo Scope", style);
	}
	
	/** {@inheritDoc} */
	@Override
	public Collection<NamedArea> getCollection(TaxonDescription entity) {
		return entity.getGeoScopes();
	}

	/** {@inheritDoc} */
	@Override
	public NamedArea createNewElement() {
		return NamedArea.NewInstance();
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(NamedArea element) {
		getEntity().addGeoScope(element);
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(NamedArea element) {
		getEntity().removeGeoScope(element);
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No geo scopes yet.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Add a new geo scope.";
	}
}
