// $Id$
/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.media;

import java.net.URI;
import java.util.List;
import java.util.Set;

import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.cdm.model.media.MediaUtils;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author pplitzner
 * @date 25.08.2014
 *
 */
public class MediaDetailElement extends AbstractCdmDetailElement<Media>{

    private UriWithLabelElement textUri;
    private MediaRepresentationSection section_mediaRepresentation;

    private boolean isAdvancedMediaView;
    private MediaRepresentationPart singleMediaRepresentationPart;
    private ICdmFormElement parentFormElement;
    private int style;

    /**
     * Used to store the URI even if it is invalid and thus cannot be stored in CDM
     */
    private String uriBuffer;

    public MediaDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, Media entity, int style) {
        this.parentFormElement = formElement;
        this.style = style;
    }

    @Override
    public void setEntity(Media entity) {
        super.setEntity(entity);
        if(singleMediaRepresentationPart==null){
            singleMediaRepresentationPart = MediaUtils.initFirstMediaRepresentationPart(entity, true);
        }
        initIsAdvancedMediaView();
        showAdvancedView();
    }

    @Override
    public void handleEvent(Object eventSource){
        if(eventSource==textUri){
            textUri.setBackground(getPersistentBackground());
            URI uri = textUri.getUri();
            singleMediaRepresentationPart.setUri(uri);
            if(uri==null){
                uriBuffer=textUri.getText();
            }
        }
    }

    public void toggleAdvancedMediaView() {
        if (getEntity().getRepresentations() != null
                && (getEntity().getRepresentations().size() > 1 ||
                        (getEntity().getRepresentations().size() == 1
                        && getEntity().getRepresentations().iterator().next().getParts().size() > 1))) {
            MessagingUtils.informationDialog("Toggling not possible",
                    "Media has consists of multiple representations or representatio parts");
            // toggling is only possible if there are no more than one
            // MediaRepresentation resp. MediaRepresentationParts
            return;
        }
        isAdvancedMediaView = !isAdvancedMediaView;
        showAdvancedView();
        reflowParentScrolledForm(true);
    }

    private void showAdvancedView() {
        if(isAdvancedMediaView){
            if(textUri!=null){
                removeElementsAndControls(textUri);
            }
            section_mediaRepresentation = formFactory.createMediaRepresentationSection(getConversationHolder(), parentFormElement, style);
            section_mediaRepresentation.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            section_mediaRepresentation.setEntity(getEntity());
            //set buffered uri as text if uri had parsing problems in simple view
            if(uriBuffer!=null){
                section_mediaRepresentation.getLayoutComposite().getChildren();
            }
        }
        else{
            if(section_mediaRepresentation!=null){
                removeElementsAndControls(section_mediaRepresentation);
            }
            textUri = formFactory.createUriWithLabelElement(parentFormElement, "Media URI", null, style);
            URI uri = singleMediaRepresentationPart.getUri();
            textUri.setUri(uri);
            //set buffered uri as text if uri had parsing problems in advanced view
            if(uri==null && uriBuffer!=null){
                textUri.setText(uriBuffer);
                textUri.getUri();
            }
            textUri.getLayoutComposite().layout();
        }
    }

    private void initIsAdvancedMediaView(){
        if(PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.SHOW_ADVANCED_MEDIA_SECTION)){
            isAdvancedMediaView = true;
            return;
        }
        Set<MediaRepresentation> representations = getEntity().getRepresentations();
        if(representations.size()>1){
            isAdvancedMediaView = true;
            return;
        }
        if(representations.size()==1){
            List<MediaRepresentationPart> parts = representations.iterator().next().getParts();
            if(parts.size()>1){
                isAdvancedMediaView =  true;
                return;
            }
        }
        isAdvancedMediaView =  false;
    }

    public boolean isAdvancedMediaView() {
        return isAdvancedMediaView;
    }

    public void setUriBuffer(String uriBuffer) {
        this.uriBuffer = uriBuffer;
    }

    public String getUriBuffer() {
        return uriBuffer;
    }

}
