// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.media;

import java.net.URI;

import org.apache.commons.io.FileUtils;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * <p>Abstract AbstractMediaRepresentationPartElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 24, 2010
 * @version 1.0
 */
public class MediaRepresentationPartElement<T extends MediaRepresentationPart> extends
		AbstractEntityCollectionElement<T> {

	protected UriWithLabelElement text_uri;
	protected NumberWithLabelElement text_size;

	public MediaRepresentationPartElement(CdmFormFactory formFactory,
			AbstractFormSection section,
			T element, SelectionListener removeListener,
			int style) {
		super(formFactory, section, element, removeListener, null, style);
	}

	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		text_uri = formFactory.createUriWithLabelElement(element, "URI", null, style);
		text_size = formFactory.createNumberTextWithLabelElement(element, "Size", null, style);
		text_size.setEnabled(false);
	}

	@Override
	public void setEntity(T entity) {
		this.entity = entity;
		if(entity.getSize() != null){
			text_size.setText(FileUtils.byteCountToDisplaySize(entity.getSize()));
		}
		if(entity.getUri() != null){
			text_uri.setUri(entity.getUri());
		} else {
            String uriBuffer = getParentMediaDetailElement().getUriBuffer();
            if(uriBuffer!=null){
                text_uri.setText(uriBuffer);
                text_uri.getUri();//just to update the error label
            }
        }
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_uri){
		    URI uri = text_uri.getUri();
		    getEntity().setUri(uri);
		    if(uri==null){
                //buffer URI if parsing error occurred
                MediaDetailElement parentMediaDetailElement = getParentMediaDetailElement();
                if(parentMediaDetailElement!=null){
                    parentMediaDetailElement.setUriBuffer(text_uri.getText());
                }
            }

		    firePropertyChangeEvent(this);
		}
	}

    protected MediaDetailElement getParentMediaDetailElement() {
        ICdmFormElement parentElement = getParentElement();
        while(parentElement!=null){
            parentElement = parentElement.getParentElement();
            if(parentElement instanceof MediaDetailElement){
                return (MediaDetailElement) parentElement;
            }
        }
        return null;
    }

}
