// $Id$
/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AbstractReferencedEntityElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 28.01.2014
 *
 */
public class SynonymRelationshipDetailElement extends AbstractReferencedEntityElement<SynonymRelationship>{

    private TextWithLabelElement text_ruleConsidered;
    private LabelElement label;
    private EntitySelectionElement selection_toName;
    private TermComboElement<SynonymRelationshipType> combo_relationshipType;
    private CheckboxElement checkProParte;
    private CheckboxElement checkPartial;

    /**
     * <p>Constructor for NameRelationshipDetailElement.</p>
     *
     * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
     * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection} object.
     * @param entity a {@link eu.etaxonomy.cdm.model.name.NameRelationship} object.
     * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
     * @param style a int.
     */
    public SynonymRelationshipDetailElement(CdmFormFactory cdmFormFactory,
            AbstractFormSection formElement,
            SynonymRelationship entity, SelectionListener removeListener, int style) {
        super(cdmFormFactory, formElement, entity, removeListener, style);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#createControls(org.eclipse.swt.widgets.Composite, int)
     */
    /** {@inheritDoc} */
    @Override
    public void createControls(ICdmFormElement element, int style) {
        label = formFactory.createLabel(element, null);
        checkProParte = formFactory.createCheckbox(element, "Pro Parte Synonym", null, style);
        checkPartial= formFactory.createCheckbox(element, "Partial Synonym", null, style);
        super.createControls(element, style);
    }

    /** {@inheritDoc} */
    @Override
    public void setEntity(SynonymRelationship entity) {
        String text = entity.getSynonym().getTitleCache() + " is " + entity.getType().getLabel() + " " + entity.getAcceptedTaxon().getTitleCache();

        label.setText(text);

        checkPartial.setSelection(entity.isPartial());
        checkProParte.setSelection(entity.isProParte());

        super.setEntity(entity);
    }

    /** {@inheritDoc} */
    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource == combo_relationshipType){
            getEntity().setType(combo_relationshipType.getSelection());
        }
        else if(eventSource == selection_toName){
            MessagingUtils.error(getClass(), "No API call available.", null);
        }
        else if(eventSource == checkPartial){
            getEntity().setPartial(checkPartial.getSelection());
        }
        else if(eventSource == checkProParte){
            getEntity().setProParte(checkProParte.getSelection());
        }
        else if(eventSource == selection_reference){
            getEntity().setCitation(selection_reference.getSelection());
        }
        else if(eventSource == text_referenceDetail){
            getEntity().setCitationMicroReference(text_referenceDetail.getText());
        }
    }

    /** {@inheritDoc} */
    @Override
    public void setBackground(Color color) {
        if(label != null) {
            label.setBackground(color);
        }
        super.setBackground(color);
    }

}
