// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.occurrence;

import java.util.Collection;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.OriginalSourceType;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * <p>SourceCollectionDetailSection class.</p>
 *
 * @author n.hoffmann
 * @created Oct 15, 2010
 * @version 1.0
 */
public class SourceCollectionDetailSection extends
		AbstractEntityCollectionSection<DerivedUnitFacade, IdentifiableSource> {

	/**
	 * <p>Constructor for SourceCollectionDetailSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public SourceCollectionDetailSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(formFactory, conversation, parentElement, "Sources", style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getCollection(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public Collection<IdentifiableSource> getCollection(DerivedUnitFacade entity) {
		return entity.getSources();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#createNewElement()
	 */
	/** {@inheritDoc} */
	@Override
	public IdentifiableSource createNewElement() {
		return IdentifiableSource.NewInstance(OriginalSourceType.PrimaryTaxonomicSource);		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#addElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	public void addElement(IdentifiableSource element) {
		getEntity().addSource(element);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#removeElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	public void removeElement(IdentifiableSource element) {
		getEntity().removeSource(element);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getEmptyString()
	 */
	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No sources yet.";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getTooltipString()
	 */
	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Add source";
	}

}
