// $Id$
/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.occurrence.association;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IParameter;
import org.eclipse.core.commands.NotEnabledException;
import org.eclipse.core.commands.NotHandledException;
import org.eclipse.core.commands.Parameterization;
import org.eclipse.core.commands.ParameterizedCommand;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author pplitzner
 * @date Dec 1, 2014
 *
 */
public class TaxonAssociationDetailElement extends AbstractCdmDetailElement<DerivedUnitFacade> implements IDoubleClickListener{


    private TableViewer associationsViewer;
    private TableViewer typeDesignationViewer;

    public TaxonAssociationDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    /** {@inheritDoc} */
    @Override
    protected void createControls(ICdmFormElement formElement, DerivedUnitFacade entity, int style) {

        //TODO add context menu for deleting associations

        Label associationsLabel = formFactory.createLabel(getLayoutComposite(), "Individuals Associations");
        associationsLabel.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));

        associationsViewer = new TableViewer(getLayoutComposite(), SWT.FULL_SELECTION);
        associationsViewer.getTable().setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        associationsViewer.setContentProvider(new ArrayContentProvider());
        Collection<IndividualsAssociation> individualsAssociations = CdmStore.getService(IOccurrenceService.class).listIndividualsAssociations(entity.innerDerivedUnit(), null, null, null, null);
        //TODO implement service method for this which is just used in the label provider
        Collection<TaxonBase<?>> associatedTaxa = new HashSet<TaxonBase<?>>();
        for (IndividualsAssociation individualsAssociation : individualsAssociations) {
            if(individualsAssociation.getInDescription().isInstanceOf(TaxonDescription.class)){
                TaxonDescription taxonDescription = HibernateProxyHelper.deproxy(individualsAssociation.getInDescription(), TaxonDescription.class);
                associatedTaxa.add(taxonDescription.getTaxon());
            }
        }
        associationsViewer.setInput(associatedTaxa);
        associationsViewer.addDoubleClickListener(this);

        Label typeLabel = formFactory.createLabel(getLayoutComposite(), "Type Designations");
        typeLabel.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));

        typeDesignationViewer = new TableViewer(getLayoutComposite(), SWT.FULL_SELECTION);
        typeDesignationViewer.getTable().setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        typeDesignationViewer.setContentProvider(new ArrayContentProvider());
        Collection<SpecimenTypeDesignation> typeDesignations = CdmStore.getService(IOccurrenceService.class).listTypeDesignations(entity.innerDerivedUnit(), null, null, null, null);
        //TODO implement service method for this which is just used in the label provider
        Collection<TaxonBase<?>> typedTaxa = new HashSet<TaxonBase<?>>();
        for (SpecimenTypeDesignation specimenTypeDesignation : typeDesignations) {
            for (TaxonNameBase taxonNameBase : specimenTypeDesignation.getTypifiedNames()) {
                Set taxa = taxonNameBase.getTaxa();
                for (Object taxon : taxa) {
                    if(taxon instanceof CdmBase && ((CdmBase)taxon).isInstanceOf(TaxonBase.class)){
                        typedTaxa.add(HibernateProxyHelper.deproxy(taxon, TaxonBase.class));
                    }
                }
            }
        }
        typeDesignationViewer.setInput(typedTaxa);
        typeDesignationViewer.addDoubleClickListener(this);
    }

    /** {@inheritDoc} */
    @Override
    public void handleEvent(Object eventSource) {
        //empty
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.IDoubleClickListener#doubleClick(org.eclipse.jface.viewers.DoubleClickEvent)
     */
    @Override
    public void doubleClick(DoubleClickEvent event) {
        if(event.getSelection() instanceof IStructuredSelection){
            Object firstElement = ((IStructuredSelection) event.getSelection()).getFirstElement();
            if(firstElement instanceof TaxonBase<?>){
                TaxonBase<?> taxonBase = (TaxonBase<?>)firstElement;
                String commandId = "eu.etaxonomy.taxeditor.editor.openTaxonEditor";


                ArrayList parameters = new ArrayList();
                IParameter iparam = null;

                //get the command from plugin.xml
                IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
                ICommandService cmdService = (ICommandService)window.getService(ICommandService.class);
                Command cmd = cmdService.getCommand(commandId);

                //get the parameter
                try {
                    iparam = cmd.getParameter("eu.etaxonomy.taxeditor.editor.taxonParameter");
                } catch (NotDefinedException e1) {
                    MessagingUtils.error(this.getClass(), "Command not defined", e1);
                }
                Parameterization params = new Parameterization(iparam, (taxonBase).getUuid().toString());
                parameters.add(params);

                //build the parameterized command
                ParameterizedCommand pc = new ParameterizedCommand(cmd, (Parameterization[]) parameters.toArray(new Parameterization[parameters.size()]));

                //execute the command
                IHandlerService handlerService = (IHandlerService)window.getService(IHandlerService.class);
                try {
                    handlerService.executeCommand(pc, null);
                } catch (ExecutionException e) {
                    MessagingUtils.error(TaxonAssociationDetailElement.class, e);
                } catch (NotDefinedException e) {
                    MessagingUtils.error(TaxonAssociationDetailElement.class, e);
                } catch (NotEnabledException e) {
                    MessagingUtils.error(TaxonAssociationDetailElement.class, e);
                } catch (NotHandledException e) {
                    MessagingUtils.error(TaxonAssociationDetailElement.class, e);
                }
            }
        }
    }

}
