// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.reference;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.reference.ReferenceFactory;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.model.AuthorHelper;
import eu.etaxonomy.taxeditor.ui.combo.EnumComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IErrorIntolerableElement;
import eu.etaxonomy.taxeditor.ui.element.IExceptionHandler;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * ReferenceBaseDetailElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Mar 2, 2010
 * @version 1.0
 */
public class ReferenceDetailElement extends AbstractIdentifiableEntityDetailElement<Reference>
		implements IErrorIntolerableElement, IExceptionHandler {

    private ToggleableTextElement toggleableAbbrevCache;

	private TextWithLabelElement text_edition;
	private TextWithLabelElement text_editor;
	private TextWithLabelElement text_isbn;
	private TextWithLabelElement text_issn;
	private TextWithLabelElement text_organisation;
	private TextWithLabelElement text_pages;
	private TextWithLabelElement text_placePublished;
	private TextWithLabelElement text_publisher;
	private TextWithLabelElement text_referenceAbstract;
	private TextWithLabelElement text_series;
	private TextWithLabelElement text_seriesPart;
	private TextWithLabelElement text_volume;
	private TextWithLabelElement text_abbrevTitle;
	private TextWithLabelElement text_title;
	private UriWithLabelElement text_uri;
	private EntitySelectionElement<AgentBase> selection_authorTeam;
	private TimePeriodElement element_timePeriod;
	private EntitySelectionElement<Reference> selection_inReference;
	private EntitySelectionElement<Institution> selection_institution;
	private EnumComboElement<ReferenceType> combo_referenceType;

	private EntitySelectionElement<Reference> selection_inSeries;

	/**
	 * <p>
	 * Constructor for ReferenceBaseDetailElement.
	 * </p>
	 *
	 * @param cdmFormFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param style
	 *            a int.
	 */
	public ReferenceDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.section.cdmdetail.AbstractCdmDetailComposite
	 * #createControl(org.eclipse.swt.widgets.Composite, int)
	 */
	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement,
			Reference entity, int style) {
		toggleable_cache = formFactory.createToggleableTextField(formElement,
				"Reference Cache", entity.getTitleCache(),
				entity.isProtectedTitleCache(), style);

		toggleableAbbrevCache = formFactory.createToggleableTextField(formElement,
		        "Abbrev. Cache", entity.getAbbrevTitleCache(),
		        entity.isProtectedAbbrevTitleCache(), style);

		combo_referenceType = formFactory.createEnumComboElement(ReferenceType.class,
						formElement, style);

		combo_referenceType.setSelection(entity.getType());

		// title
		text_title = formFactory.createTextWithLabelElement(formElement,
				"Title", entity.getTitle(), style);
		// abbreviated title
		text_abbrevTitle = formFactory.createTextWithLabelElement(formElement,
		        "Abbrev. Title", entity.getAbbrevTitle(), style);

		// author team
		selection_authorTeam = formFactory
				.createSelectionElement(AgentBase.class,
						getConversationHolder(), formElement, "Author",
						AuthorHelper.getAuthor(entity.getAuthorship()),
						EntitySelectionElement.ALL, style);

		ReferenceType referenceType = entity.getType();

		if (referenceType != null) {

			createInReferenceByType(this, referenceType, entity, SWT.NULL);

			if (referenceType.equals(ReferenceType.Generic)) {
				createGenericControls(this, entity, SWT.NULL);
				createPublicationControls(this, referenceType, entity, SWT.NULL);
			} else {
				if (referenceType.isPrintedUnit()) {
					createPrintedUnitControls(this, referenceType, entity,
							SWT.NULL);
				}
				if (referenceType.isPublication()) {
					createPublicationControls(this, referenceType, entity,
							SWT.NULL);
				}
				if (referenceType.isVolumeReference()) {
					createVolumeControls(this, entity, SWT.NULL);
				}
				if (referenceType.isSection()) {
					createSectionControls(this, entity, SWT.NULL);
				}
			}
		}

		// date published
		element_timePeriod = formFactory.createTimePeriodElement(formElement,
				"Date Published", entity.getDatePublished(), style);

		createUriAndAbstract(this, entity, SWT.NULL);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.section.cdmdetail.AbstractCdmDetailComposite
	 * #updateContent()
	 */
	/** {@inheritDoc} */
	@Override
	protected void updateContent() {
		if (getEntity() == null) {
			setEntity(ReferenceFactory.newGeneric());
		}

		super.updateContent();

		// enable elements according to state of cache protection
		// setEnabled(getEntity().isProtectedTitleCache(), Arrays.asList(new
		// Object[]{text_cache}));
		toggleable_cache.setEnabled(getEntity().isProtectedTitleCache());
		toggleableAbbrevCache.setEnabled(getEntity().isProtectedAbbrevTitleCache());
        setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[] { toggleable_cache }));
        setIrrelevant(toggleableAbbrevCache.getState(), Arrays.asList(new Object[] { toggleableAbbrevCache}));
	}

	/**
	 * Creates an in reference for reference types that do hav this.
	 *
	 * @param element
	 * @param referenceType
	 * @param reference
	 * @param style
	 */
	private void createInReferenceByType(ICdmFormElement element,
			ReferenceType referenceType, Reference reference, int style) {
		boolean createInReferenceWidget = true;
		String label = "In Reference";

		switch (referenceType) {
		case Generic:
			break;
		case Section:
		    break;
		case BookSection:
			label = "Book";
			break;
		case InProceedings:
			label = "Print Series";
			break;
		case Article:
			label = "Journal";
			break;
		case WebPage:
			break;
		default:
			createInReferenceWidget = false;
		}

		if (createInReferenceWidget) {
			selection_inReference = formFactory
					.createSelectionElement(Reference.class,
							getConversationHolder(), element, label,
							reference.getInReference(),
							EntitySelectionElement.ALL, style);
		}
	}

	/**
	 * Creates additional controls to edit a generic reference
	 *
	 * @param element
	 * @param reference
	 * @param listener
	 * @param style
	 */
	private void createGenericControls(ICdmFormElement element,
			Reference reference, int style) {

		// editor
		text_editor = formFactory.createTextWithLabelElement(element, "Editor",
				reference.getEditor(), style);

		// series
		text_series = formFactory.createTextWithLabelElement(element, "Series",
				reference.getSeriesPart(), style);

		// volume
		text_volume = formFactory.createTextWithLabelElement(element, "Volume",
				reference.getVolume(), style);

		// pages
		text_pages = formFactory.createTextWithLabelElement(element, "Pages",
				reference.getPages(), style);
	}

	/**
	 * Creates additional controls to edit a publication
	 *
	 * @param element
	 * @param reference
	 * @param listener
	 * @param style
	 */
	private void createPublicationControls(ICdmFormElement element,
			ReferenceType referenceType, Reference reference, int style) {

		// placePublished
		text_placePublished = formFactory.createTextWithLabelElement(element,
				"Place Published", reference.getPlacePublished(), style);

		// publisher
		text_publisher = formFactory.createTextWithLabelElement(element,
				"Publisher", reference.getPublisher(), style);

		if (referenceType.equals(ReferenceType.Journal)) {
			// issn
			text_issn = formFactory.createTextWithLabelElement(element, "ISSN",
					reference.getIssn(), style);
		}

		if (referenceType.equals(ReferenceType.Report)) {
			// institution
			selection_institution = formFactory
					.createSelectionElement(Institution.class,
							getConversationHolder(), element, "Institution",
							reference.getInstitution(),
							EntitySelectionElement.ALL, style);
		}

		if (referenceType.equals(ReferenceType.Thesis)) {
			// school
			selection_institution = formFactory
					.createSelectionElement(Institution.class,
							getConversationHolder(), element, "School",
							reference.getSchool(),
							EntitySelectionElement.ALL, style);
		}

		if (referenceType.equals(ReferenceType.Proceedings)) {
			// organization
			text_organisation = formFactory.createTextWithLabelElement(element,
					"Organisation", reference.getOrganization(), style);
		}
	}

	/**
	 * Creates additional controls to edit a section
	 *
	 * @param element
	 * @param reference
	 * @param style
	 */
	private void createSectionControls(ICdmFormElement element,
			Reference reference, int style) {
		// pages
		text_pages = formFactory.createTextWithLabelElement(element, "Pages",
				reference.getPages(), style);
	}

	/**
	 * Creates additional controls to edit a volume
	 *
	 * @param element
	 * @param reference
	 * @param style
	 */
	private void createVolumeControls(ICdmFormElement element,
			Reference reference, int style) {

		text_volume = formFactory.createTextWithLabelElement(element, "Volume",
				reference.getVolume(), style);

		text_series = formFactory.createTextWithLabelElement(element, "Series",
				reference.getSeriesPart(), style);
	}

	/**
	 *
	 * @param element
	 * @param referenceType
	 * @param reference
	 * @param style
	 */
	private void createPrintedUnitControls(ICdmFormElement element,
			ReferenceType referenceType, Reference reference, int style) {
		// in series
		selection_inSeries = formFactory
				.createSelectionElement(Reference.class,
						getConversationHolder(), element, "In Series",
						(Reference) reference.getInSeries(),
						EntitySelectionElement.ALL, style);

		// editor
		text_editor = formFactory.createTextWithLabelElement(element, "Editor",
				reference.getEditor(), style);

		// series part
		text_seriesPart = formFactory.createTextWithLabelElement(element,
				"Series", reference.getSeriesPart(), style);

		if (referenceType.equals(ReferenceType.Book)) {
			// edition
			text_edition = formFactory.createTextWithLabelElement(element,
					"Edition", reference.getEdition(), style);

			// isbn
			text_isbn = formFactory.createTextWithLabelElement(element, "ISBN",
					reference.getIsbn(), style);
		}
	}

	private void createUriAndAbstract(ICdmFormElement element,
			Reference reference, int style) {
		// uri
		text_uri = formFactory.createUriWithLabelElement(element, "URI",
				reference.getUri(), style);
		// abstract
		text_referenceAbstract = formFactory.createMultiLineTextWithLabel(
				element, "Reference Abstract", 100, style);
		text_referenceAbstract.setText(reference.getReferenceAbstract());
	}

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == combo_referenceType) {
			getEntity().setType(combo_referenceType.getSelection());
			updateContent();
		} else if (eventSource == element_timePeriod) {
			getEntity().setDatePublished(element_timePeriod.getTimePeriod());
			clearException();
		}
		// selections
		else if (eventSource == selection_authorTeam) {
			getEntity().setAuthorship(
					(TeamOrPersonBase) selection_authorTeam.getSelection());
		} else if (eventSource == selection_inReference) {
			getEntity().setInReference(selection_inReference.getSelection());
		} else if (eventSource == selection_inSeries) {
			getEntity().setInReference(selection_inSeries.getSelection());
		} else if (eventSource == selection_institution) {
			getEntity().setInstitution(selection_institution.getSelection());
		}

		// textfields
		else if (eventSource == toggleable_cache) {
			handleToggleableCacheField();
		}
		else if (eventSource == toggleableAbbrevCache) {
		    handleToggleableAbbrevTitleField();
		} else if (eventSource == text_edition) {
			getEntity().setEdition(text_edition.getText());
		} else if (eventSource == text_editor) {
			getEntity().setEditor(text_editor.getText());
		} else if (eventSource == text_isbn) {
			getEntity().setIsbn(text_isbn.getText());
		} else if (eventSource == text_issn) {
			getEntity().setIssn(text_issn.getText());
		} else if (eventSource == text_organisation) {
			getEntity().setOrganization(text_organisation.getText());
		} else if (eventSource == text_pages) {
			getEntity().setPages(text_pages.getText());
		} else if (eventSource == text_placePublished) {
			getEntity().setPlacePublished(text_placePublished.getText());
		} else if (eventSource == text_publisher) {
			getEntity().setPublisher(text_publisher.getText());
		} else if (eventSource == text_referenceAbstract) {
			getEntity().setReferenceAbstract(text_referenceAbstract.getText());
		} else if (eventSource == text_seriesPart) {
			getEntity().setSeriesPart(text_seriesPart.getText());
		} else if (eventSource == text_title) {
			getEntity().setTitle(text_title.getText());
			toggleable_cache.setText(text_title.getText());
		} else if (eventSource == text_abbrevTitle) {
		    getEntity().setAbbrevTitle(text_abbrevTitle.getText());
		    toggleableAbbrevCache.setText(text_abbrevTitle.getText());
		} else if (eventSource == text_uri) {
			getEntity().setUri(text_uri.getUri());
		} else if (eventSource == text_volume) {
			getEntity().setVolume(text_volume.getText());
		}

		if (eventSource != text_title && eventSource != text_abbrevTitle) {
		    // set abbrev title cache to null to force new generation
		    // for the standard title cache this is done by a PropertyChangeListener registered
		    // in IdentifiableEntity.initListeners()
		    if(!getEntity().isProtectedAbbrevTitleCache()){
		        getEntity().setAbbrevTitleCache(null, getEntity().isProtectedAbbrevTitleCache());
		    }
			toggleable_cache.setText(getEntity().getTitleCache());
			toggleableAbbrevCache.setText(getEntity().getAbbrevTitleCache());
		}

		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}


    @Override
    public void updateToggleableCacheField() {
        if (!getEntity().isProtectedTitleCache()) {
            toggleable_cache.setText(getEntity().generateTitle());
        }
        if (!getEntity().isProtectedAbbrevTitleCache()) {
            toggleableAbbrevCache.setText(getEntity().generateAbbrevTitle());
        }
    }

    protected void handleToggleableAbbrevTitleField() {
        boolean pushedAbbrevState = toggleableAbbrevCache.getState();
        getEntity().setAbbrevTitleCache(toggleableAbbrevCache.getText(), pushedAbbrevState);
        setIrrelevant(pushedAbbrevState, Arrays.asList(new Object[] { toggleableAbbrevCache, toggleable_cache, text_title }));
        updateToggleableCacheField();
    }

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleException
	 * (eu.etaxonomy.taxeditor.forms.CdmPropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleException(CdmPropertyChangeEvent event) {
		exceptionOccurred(event);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.forms.IExceptionHandler#clearException()
	 */
	/** {@inheritDoc} */
	@Override
	public void clearException() {
		exceptionOccurred(null);
	}

	private final List<IExceptionHandler> exceptionHandlers = new ArrayList<IExceptionHandler>();

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.IErrorIntolerableElement#addExceptionHandler
	 * (eu.etaxonomy.taxeditor.forms.IExceptionHandler)
	 */
	/** {@inheritDoc} */
	@Override
	public void addExceptionHandler(IExceptionHandler handler) {
		exceptionHandlers.add(handler);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.IErrorIntolerableElement#removeExceptionHandler
	 * (eu.etaxonomy.taxeditor.forms.IExceptionHandler)
	 */
	/** {@inheritDoc} */
	@Override
	public void removeExceptionHandler(IExceptionHandler handler) {
		exceptionHandlers.remove(handler);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.IErrorIntolerableElement#exceptionOccurred
	 * (eu.etaxonomy.taxeditor.forms.CdmPropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	@Override
	public void exceptionOccurred(CdmPropertyChangeEvent event) {
		for (IExceptionHandler handler : exceptionHandlers) {
			if (event == null) {
				handler.clearException();
			} else {
				handler.handleException(event);
			}
		}
	}
}
