/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.cdm.model.media.RightsType;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * RightsElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Nov 16, 2009
 * @version 1.0
 */
public class RightsElement extends AbstractEntityCollectionElement<Rights> {

	private TermComboElement<RightsType> combo_rightsType;
	private EntitySelectionElement<AgentBase> selection_agent;
	private TextWithLabelElement text;

	/**
	 * <p>
	 * Constructor for RightsElement.
	 * </p>
	 *
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection}
	 *            object.
	 * @param element
	 *            a {@link eu.etaxonomy.cdm.model.media.Rights} object.
	 * @param removeListener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style
	 *            a int.
	 * @param cdmFormFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 */
	public RightsElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection formElement, Rights element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, element, removeListener, null, style);
	}


	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite
	 * #createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		combo_rightsType = formFactory.createDefinedTermComboElement(TermType.RightsType, this, "Rights Type", null, style);
		selection_agent = formFactory.createSelectionElement(
				AgentBase.class, getConversationHolder(), this, "Agent",
				null, EntitySelectionElement.ALL, style);
		text = formFactory.createMultiLineTextWithLabel(this, "Text", 100,
				style);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite
	 * #setElement(eu.etaxonomy.cdm.model.common.VersionableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	public void setEntity(Rights element) {
		this.entity = element;
		combo_rightsType.setSelection(element.getType());
		selection_agent.setEntity(element.getAgent());
		text.setText(element.getText());
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent
	 * (java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == combo_rightsType) {
			entity.setType(combo_rightsType.getSelection());
		} else if (eventSource == selection_agent) {
			entity.setAgent(selection_agent.getSelection());
		} else if (eventSource == text) {
			entity.setText(text.getText());
		}
	}

}
