// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.taxon;

import java.util.Arrays;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.ui.forms.widgets.ExpandableComposite;

import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.hibernate.permission.Role;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;
import eu.etaxonomy.taxeditor.ui.section.name.SynonymRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * TaxonBaseDetailElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Feb 26, 2010
 * @version 1.0
 */
public class TaxonBaseDetailElement extends AbstractIdentifiableEntityDetailElement<TaxonBase> {

	private CheckboxElement checkbox_doubtful;

	private CheckboxElement checkbox_useNameCache;

	private EntitySelectionElement<Reference> selection_secundum;

	private TextWithLabelElement text_appendedPhrase;

	private CheckboxElement checkbox_excluded;

	private CheckboxElement checkbox_unplaced;

	private CheckboxElement checkbox_published;

	private SynonymRelationshipDetailSection sectionSynonymRelationship;

	/**
	 * <p>
	 * Constructor for TaxonBaseDetailElement.
	 * </p>
	 *
	 * @param cdmFormFactory
	 *        a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement
	 *        a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style
	 *            a int.
	 */
	public TaxonBaseDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}

	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement, TaxonBase entity, int style) {
			toggleable_cache = formFactory.createToggleableTextField(formElement,
					"Title Cache", entity.getTitleCache(),
					entity.isProtectedTitleCache(), style);

		//TODO RL
		boolean rl=true;
		if(PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.IS_RL)){
			toggleable_cache.setVisible(false);
		}
		if (!PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.IS_RL)){
			text_appendedPhrase = formFactory.createTextWithLabelElement(
					formElement, "Appended Phrase", entity.getAppendedPhrase(),
					SWT.NULL);			
		}

		selection_secundum = formFactory.createSelectionElement(Reference.class,
				getConversationHolder(), formElement, "Secundum",
				entity.getSec(), EntitySelectionElement.ALL,
				style);
		
		// TODO RL
		if (!PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.IS_RL)) {
			checkbox_useNameCache = formFactory.createCheckbox(formElement,
					"Exclude Authorship", entity.isUseNameCache(), style);
	
			if (entity instanceof Taxon) {
				checkbox_excluded = formFactory.createCheckbox(formElement,
						"Taxon is excluded", ((Taxon) entity).isExcluded(), style);
				checkbox_unplaced = formFactory.createCheckbox(formElement,
						"Taxon is unplaced", ((Taxon) entity).isUnplaced(), style);
			}
	
			checkbox_doubtful = formFactory.createCheckbox(formElement,
					entity.getClass().getSimpleName() + " is doubtful", entity.isDoubtful(), style);
		}
		
		if (entity instanceof Taxon) {
			checkbox_published = formFactory.createCheckbox(formElement,
					"Taxon is published", ((Taxon) entity).isPublish(), style);
		}


		if(entity instanceof Synonym){
		    sectionSynonymRelationship = formFactory.createSynonymRelationshipDetailSection(getConversationHolder(), formElement, ExpandableComposite.TWISTIE);
		    sectionSynonymRelationship.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
		    sectionSynonymRelationship.setEntity((Synonym) entity);
		}
	}

    private void updateCheckboxPublish() {
        if(getEntity() instanceof Taxon){
            checkbox_published.setEnabled(
            		CdmStore.currentAuthentiationHasOneOfRoles(Role.ROLE_PUBLISH, Role.ROLE_ADMIN)
            		);
        }
    }

	@Override
    protected void updateControlStates() {
        super.updateControlStates();
        updateCheckboxPublish();
    }

    /** {@inheritDoc} */
	@Override
	protected void updateContent() {
		super.updateContent();
		toggleable_cache.setEnabled(getEntity().isProtectedTitleCache());
		setIrrelevant(toggleable_cache.getState(),
				Arrays.asList(new Object[] { toggleable_cache }));
	}

	@Override
	protected void handleTitleCacheRelevantChange(PropertyChangeEvent event) {
		String title = getEntity().generateTitle();
		getEntity().setTitleCache(title);
		super.handleTitleCacheRelevantChange(event);
	}

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource== toggleable_cache) {
			handleToggleableCacheField();
		} else if (eventSource == checkbox_doubtful) {
			getEntity().setDoubtful(checkbox_doubtful.getSelection());
		} else if (eventSource == checkbox_useNameCache) {
			getEntity().setUseNameCache(checkbox_useNameCache.getSelection());
		} else if (eventSource == selection_secundum) {
			getEntity().setSec(selection_secundum.getSelection());
		} else if (eventSource == text_appendedPhrase) {
			getEntity().setAppendedPhrase(text_appendedPhrase.getText());
		} else if (eventSource == checkbox_excluded) {
			((Taxon) getEntity()).setExcluded(checkbox_excluded.getSelection());
		} else if (eventSource == checkbox_unplaced) {
		    ((Taxon) getEntity()).setUnplaced(checkbox_unplaced.getSelection());
		} else if (eventSource == checkbox_published) {
		    ((Taxon)getEntity()).setPublish(checkbox_published.getSelection());
		}
	}
}
