// $Id$
/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.Representation;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.taxeditor.editor.definedterm.TermBasePropertyTester;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author l.morris
 * @date 20 Dec 2011
 *
 */
public abstract class AbstractTermBaseDetailElement<T extends TermBase> extends AbstractCdmDetailElement<T> {

	protected TextWithLabelElement text_label;
	protected TextWithLabelElement text_description;
	protected UriWithLabelElement uri_uri;
	protected TextWithLabelElement text_abbreviatedLabel;
	
	/**
	 * @param formFactory
	 * @param formElement
	 */
	public AbstractTermBaseDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}
	
	@Override
	protected void createControls(ICdmFormElement formElement,
			T entity, int style) {
		
		Representation representation = getEntity().getPreferredRepresentation(CdmStore.getDefaultLanguage());
		
		text_label = formFactory.createTextWithLabelElement(formElement, "Label", representation.getLabel(), style);
		text_abbreviatedLabel = formFactory.createTextWithLabelElement(formElement, "Abbrev. Label", representation.getAbbreviatedLabel(), style);
		text_description = formFactory.createMultiLineTextWithLabel(formElement, "Description", 100, style);
		text_description.setText(representation.getDescription());
		uri_uri = formFactory.createUriWithLabelElement(formElement, "URI", getEntity().getUri(), style);
		
	}

	@Override
	public void setEntity(T entity) {
		super.setEntity(entity);
		setEnabled(TermBasePropertyTester.isModifiable(entity));
	}
	
	protected void handleRepresentation(Object eventSource){
		T entity = getEntity();
		Representation representation = entity.getRepresentation(CdmStore.getDefaultLanguage());
		
		if (representation == null){
			return;
		}
		
		if (eventSource == text_label){
			representation.setLabel(text_label.getText());
			entity.setLabel(text_label.getText());
		}else if (eventSource == text_abbreviatedLabel){
			representation.setAbbreviatedLabel(text_abbreviatedLabel.getText());
		}else if (eventSource == text_description){
			representation.setText(text_description.getText());
		}
	}

	public abstract TermVocabulary getVocabulary();
		
}
