// $Id$
/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.view.derivateSearch;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.wb.swt.ResourceManager;

/**
 * The widgets of the {@link DerivateSearchView}<br>
 *
 */
public class DerivateSearchComposite extends Composite {
    private final FormToolkit formToolkit = new FormToolkit(Display.getDefault());
    private final Text searchField;
    private final Table table;
    private final TableViewer resultViewer;
    private final Combo comboDerivateType;
    private final Button buttonSearch;
    private final Button btnFilterUndeterminedSpecimen;
    private final Text textTaxonName;
    private final Button btnBrowseTaxa;
    private final Label lblTaxon;
    private final Label lblDerivateType;
    private final Button btnClearTaxon;

    /**
     * Create the composite.
     * @param parent
     * @param style
     */
    public DerivateSearchComposite(Composite parent, int style) {
        super(parent, style);
        setLayout(new GridLayout(4, false));

        lblTaxon = new Label(this, SWT.NONE);
        lblTaxon.setText("Taxon");

        textTaxonName = formToolkit.createText(this, "New Text", SWT.NONE);
        textTaxonName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        textTaxonName.setEnabled(false);
        textTaxonName.setText("");

        btnBrowseTaxa = formToolkit.createButton(this, "", SWT.NONE);
        btnBrowseTaxa.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/open.gif"));

        btnClearTaxon = formToolkit.createButton(this, "", SWT.NONE);
        btnClearTaxon.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
        btnClearTaxon.setImage(ResourceManager.getPluginImage("eu.etaxonomy.taxeditor.store", "icons/trash.gif"));

        lblDerivateType = new Label(this, SWT.NULL);
        lblDerivateType.setText("Derivative Type");

        comboDerivateType = new Combo(this, SWT.READ_ONLY);
        comboDerivateType.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        formToolkit.paintBordersFor(comboDerivateType);

        btnFilterUndeterminedSpecimen = new Button(this, SWT.CHECK);
        btnFilterUndeterminedSpecimen.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
        btnFilterUndeterminedSpecimen.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
            }
        });
        btnFilterUndeterminedSpecimen.setText("Determined");

                buttonSearch = new Button(this, SWT.NONE);
                formToolkit.adapt(buttonSearch, true, true);
                buttonSearch.setText("Search");

                searchField = formToolkit.createText(this, "New Text", SWT.NONE);
                searchField.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
                searchField.setText("");
                new Label(this, SWT.NONE);
                new Label(this, SWT.NONE);

                resultViewer = new TableViewer(this, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
                table = resultViewer.getTable();
                table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 4, 1));

    }

    @Override
    protected void checkSubclass() {
        // Disable the check that prevents subclassing of SWT components
    }
    public Text getSearchField() {
        return searchField;
    }
    public TableViewer getResultViewer() {
        return resultViewer;
    }
    public Combo getComboDerivateType() {
        return comboDerivateType;
    }
    public Button getButtonSearch() {
        return buttonSearch;
    }

    @Override
    public void setEnabled(boolean enabled){
        super.setEnabled(enabled);
        searchField.setEnabled(enabled);
        table.setEnabled(enabled);
        //        resultViewer.setEnabled(enabled);
        comboDerivateType.setEnabled(enabled);
        buttonSearch.setEnabled(enabled);
        btnFilterUndeterminedSpecimen.setEnabled(enabled);
        btnBrowseTaxa.setEnabled(enabled);
        lblTaxon.setEnabled(enabled);
        lblDerivateType.setEnabled(enabled);
    }
    public Button getBtnFilterUndeterminedSpecimen() {
        return btnFilterUndeterminedSpecimen;
    }
    public Button getBtnBrowseTaxa() {
        return btnBrowseTaxa;
    }
    public Text getTextTaxonName() {
        return textTaxonName;
    }
    public Button getBtnClearTaxon() {
        return btnClearTaxon;
    }
}
