// $Id$
/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.view.derivateSearch;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * This view allows to search for and filter {@link SpecimenOrObservationBase}s and display
 * the results in a list. The results can be selected and opened in an editor.
 */
public class DerivateSearchView extends ViewPart implements IContextListener {

    public static final String ID = "eu.etaxonomy.taxeditor.view.derivateSearch.DerivateSearchView";
    private DerivateSearchCompositeController derivateSearchCompositeController;
    private ConversationHolder conversationHolder;

    /**
     * Constructs a new DerivateSearchView and registers it to listen to context changes
     */
    public DerivateSearchView() {
        CdmStore.getContextManager().addContextListener(this);
    }

    @Override
    public void createPartControl(Composite parent) {
        derivateSearchCompositeController = new DerivateSearchCompositeController(parent, this);
        getSite().setSelectionProvider(derivateSearchCompositeController.getResultViewer());
        derivateSearchCompositeController.setEnabled(CdmStore.isActive());

        //create context menu
        MenuManager menuManager = new MenuManager();
        getSite().registerContextMenu(menuManager, derivateSearchCompositeController.getResultViewer());
        Control control = derivateSearchCompositeController.getResultViewer().getControl();
        Menu menu = menuManager.createContextMenu(control);
        control.setMenu(menu);
    }

    @Override
    public void setFocus() {
        derivateSearchCompositeController.setFocus();
        //make sure to bind again if maybe in another view the conversation was unbound
        if(getConversationHolder()!=null && !getConversationHolder().isClosed() && !getConversationHolder().isBound()){
            getConversationHolder().bind();
        }
    }

    @Override
    public void contextAboutToStop(IMemento memento, IProgressMonitor monitor) {
    }

    @Override
    public void contextStop(IMemento memento, IProgressMonitor monitor) {
        derivateSearchCompositeController.setEnabled(false);
        derivateSearchCompositeController.reset();
    }

    @Override
    public void contextStart(IMemento memento, IProgressMonitor monitor) {
        derivateSearchCompositeController.setEnabled(true);
    }

    @Override
    public void contextRefresh(IProgressMonitor monitor) {
        initConversation();
    }

    private void initConversation(){
        if(conversationHolder==null){
            conversationHolder = CdmStore.createConversation();
            derivateSearchCompositeController.setConversation(conversationHolder);
        }
    }

    @Override
    public void workbenchShutdown(IMemento memento, IProgressMonitor monitor) {
    }

    public ConversationHolder getConversationHolder() {
        if(CdmStore.isActive() && conversationHolder==null){
            initConversation();
        }
        return conversationHolder;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void dispose() {
        if(conversationHolder!=null){
            conversationHolder.close();
        }
    }
}
