
// $Id$
/**
 * Copyright (C) 2011 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.view.detail;

import java.util.Set;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.ExpandableComposite;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.ext.occurrence.gbif.GbifResponse;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.description.SpecimenDescription;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.MediaSpecimen;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.IDerivedUnitFacadePart;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.model.PolytomousKeyRelationship;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.UsageTermCollection;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.RootElement;
import eu.etaxonomy.taxeditor.ui.section.agent.PersonDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamOrPersonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.common.ReferencedEntityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescribedSpecimenSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementMediaSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementSourceSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionSourceSection;
import eu.etaxonomy.taxeditor.ui.section.description.MediaDetailsSection;
import eu.etaxonomy.taxeditor.ui.section.description.NaturalLanguageSection;
import eu.etaxonomy.taxeditor.ui.section.description.ScopeSection;
import eu.etaxonomy.taxeditor.ui.section.feature.FeatureDistributionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.group.GroupDetailSection;
import eu.etaxonomy.taxeditor.ui.section.group.MemberDetailSection;
import eu.etaxonomy.taxeditor.ui.section.identifier.IdentifierDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyNodeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.TaxonomicScopeSection;
import eu.etaxonomy.taxeditor.ui.section.name.NameRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.NomenclaturalStatusSection;
import eu.etaxonomy.taxeditor.ui.section.name.NonViralNameDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.ProtologueSection;
import eu.etaxonomy.taxeditor.ui.section.name.TypeDesignationSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DeterminationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.EmptySection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldUnitGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GatheringEventDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.association.TaxonAssociationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaQualityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaSampleGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.DnaSamplePreparationPreservationSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SampleDesignationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceContigFileCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SequenceReferenceCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SingleReadGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.SingleReadPherogramCollectionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.TissueSampleGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.media.MediaSpecimenGeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.NomenclaturalReferenceDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceDetailSection;
import eu.etaxonomy.taxeditor.ui.section.supplemental.RightsSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.ParsingMessagesSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.user.GroupsByUserDetailSection;
import eu.etaxonomy.taxeditor.ui.section.user.UserDetailSection;
import eu.etaxonomy.taxeditor.ui.section.userecords.UseRecordDetailSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.TermVocabularyDetailSection;
import eu.etaxonomy.taxeditor.view.AbstractCdmDataViewer;
import eu.etaxonomy.taxeditor.view.AbstractCdmViewPart;

/**
 *
 * The DetailsViewer handles the content of the details view (
 * {@link DetailsViewPart}).<br>
 * Depending on the type of the selected element the section of the details view
 * are created.
 *
 * @author n.hoffmann
 * @created Feb 12, 2010
 * @version 1.0
 */
public class DetailsViewer extends AbstractCdmDataViewer {

    private ISelection selection;

    /**
     * <p>
     * Constructor for DetailsViewer.
     * </p>
     *
     * @param parent
     *            a {@link org.eclipse.swt.widgets.Composite} object.
     * @param viewPart
     *            a {@link eu.etaxonomy.taxeditor.view.AbstractCdmViewPart}
     *            object.
     */
    public DetailsViewer(Composite parent, AbstractCdmViewPart viewPart) {
        super(parent, viewPart);
    }

    // START HACK TO MAKE THE DERIVED UNIT FACADE WORK
    // since we are getting implementations of DerivedUnitBase from the bulk
    // editor
    // and not derived unit facade objects,

    @Override
    public void setInput(Object input) {
        if(input instanceof TreeNode){
            input = ((TreeNode) input).getValue();
        }
        if(input instanceof GbifResponse){
            input = ((GbifResponse) input).getDerivedUnitFacade();
        }
        if (input.getClass().equals(DerivedUnit.class)) {
            DerivedUnit derivedUnit = (DerivedUnit)input;
            try {
                input = DerivedUnitFacade.NewInstance(derivedUnit,
                        PreferencesUtil.getDerivedUnitConfigurator());
            } catch (DerivedUnitFacadeNotSupportedException e) {
//                MessagingUtils.error(getClass(), e); //nothing should happen, later on the Details View will show an according information that it is not possible to open this data
            }
        }
        else if(input instanceof FieldUnit){
            input = DerivedUnitFacade.NewInstance(SpecimenOrObservationType.FieldUnit, (FieldUnit) input);
        }
        super.setInput(input);
    }

    @Override
    protected void markViewPartDirty() {
        if (getInput() instanceof DerivedUnitFacade) {
            DerivedUnitFacade facade = (DerivedUnitFacade) getInput();
            if(facade.innerDerivedUnit()!=null){
                getViewPart().changed(facade.innerDerivedUnit());
            }
            else if(facade.innerFieldUnit()!=null){
                getViewPart().changed(facade.innerFieldUnit());
            }
        }
        else{
            super.markViewPartDirty();
        }
    }

    // END HACK TO MAKE THE DERIVED UNIT FACADE WORK


    /** {@inheritDoc} */
    @Override
    protected void showParts() {
        // FIXME (CM) : Need to clean out this code.
        // Too much type checking to decide which detail view to display.
        // Need to build in a mechanism where navigators / editors are 'aware'
        // of the corresponding detail viewer.
        Object input = getInput();

        destroySections();
        if (input instanceof TaxonBase) {
            if(((TaxonBase) input).getName()==null){
                createEmptySection(rootElement);
            }
            else{
                createTaxonSections(rootElement);
            }

        } else if (input instanceof NonViralName) {
            createNameSections(rootElement);

        } else if (input instanceof Reference) {
            createReferenceSections(rootElement);

        } else if (input instanceof Team) {
            createTeamDetailSection(rootElement);

        } else if (input instanceof Person) {
            createPersonDetailSection(rootElement);

        } else if (input instanceof TeamOrPersonBase) {
            createTeamOrPersonBaseDetailSection(rootElement);

        } else if (input instanceof DescriptionBase) {
            if(input instanceof SpecimenDescription){
                //TODO: add more sections to DetailsView for SpecimenDescription
                createSpecimenDescriptionSection(rootElement);
            }
            else{
                Set<Marker> descriptionMarkers = ((DescriptionBase<?>) input).getMarkers();

                Boolean isUseDescription = false;
                for (Marker marker : descriptionMarkers) {
                    if (marker.getMarkerType()!= null && marker.getMarkerType().getUuid().equals(UsageTermCollection.uuidUseMarkerType)) {
                        isUseDescription = true;
                    }
                }
                if (((DescriptionBase<?>) input).isImageGallery()) {
                    createImageGallerySection(rootElement);

                } else if (isUseDescription) {
                    createUseDescriptionSection(rootElement);
                } else {
                    createDescriptionSection(rootElement);

                }
            }
        } else if (input instanceof DescriptionElementBase) {
            Set<Marker> descriptionMarkers = ((DescriptionElementBase) input).getInDescription().getMarkers();

            Boolean isUseDescription = false;
            for (Marker marker : descriptionMarkers) {
                if (marker.getMarkerType()!= null && marker.getMarkerType().getUuid().equals(UsageTermCollection.uuidUseMarkerType)) {
                    isUseDescription = true;
                }
            }

            if (isUseDescription == true && input instanceof CategoricalData) {
                createUseRecordSection(rootElement);
            } else {
                createDescriptionElementSection(rootElement);
            }

        } else if (input instanceof Media) {
            createMediaElementSection(rootElement);

        } else if (input instanceof DerivedUnitFacade) {
            if(((DerivedUnitFacade) input).getType()==SpecimenOrObservationType.FieldUnit){
                if(((DerivedUnitFacade) input).innerFieldUnit()==null){
                    MessagingUtils.error(DetailsViewer.class, "FieldUnit of facade is null", null);
                }
                else{
                    createFieldUnitSection(rootElement);
                }
            }
            else if(((DerivedUnitFacade) input).getType()==SpecimenOrObservationType.PreservedSpecimen){
                if(((DerivedUnitFacade) input).innerDerivedUnit()==null){
                    MessagingUtils.error(DetailsViewer.class, "DerivedUnit of facade is null", null);
                }
                else{
                    createDerivedUnitBaseElementSection(rootElement);
                }
            }
            else if(((DerivedUnitFacade) input).getType()==SpecimenOrObservationType.TissueSample){
                //TissueSample should only be created by using it's own class
                //in future using only one class with different SpecimenOrObservationTypes is desired
//                createTissueSampleSection(rootElement);
            }
            else{
                createDerivedUnitBaseElementSection(rootElement);
            }
        } else if (input instanceof DnaSample){
            DnaSample dnaSample = (DnaSample)input;
            if(dnaSample.getRecordBasis()==SpecimenOrObservationType.TissueSample){
                createTissueSampleSection(rootElement);
            }
            else if(dnaSample.getRecordBasis()==SpecimenOrObservationType.DnaSample){
                createDnaSampleSection(rootElement);
            }
        } else if (input instanceof MediaSpecimen){
            createMediaSpecimenSection(rootElement);
        }
        else if(input instanceof Sequence){
            createSequenceSection(rootElement);
        }
        else if(input instanceof SingleRead){
            createSingleReadSection(rootElement);
        }
        else if (input instanceof FeatureNodeContainer) {
            createFeatureDistributionSection(rootElement);

        }else if ((input instanceof PolytomousKeyNode) || (input instanceof PolytomousKeyRelationship)) {
            createPolytomousKeyNodeSection(rootElement);
        } else if (input instanceof User) {
            createUserSection(rootElement);
        } else if (input instanceof Group) {
            createGroupSection(rootElement);
        } else if (input instanceof TaxonRelationship) {
            createTaxonRelationshipSection(rootElement);
        } else if (input instanceof TermVocabulary) {
            createTermVocabularySection(rootElement);
        } else if (input instanceof DefinedTermBase) {
            createDefinedTermSection(rootElement);
        }
        else {
            createEmptySection(rootElement);
        }
        layout();
    }


    /**
     * @param rootElement
     */
    private void createEmptySection(RootElement parent) {
        destroySections();

        EmptySection emptySection = formFactory.createEmptySection(formFactory, parent, SWT.NONE);

        addPart(emptySection);
    }

    /**
     * @param rootElement
     */
    private void createGroupSection(RootElement parent) {
        destroySections();

        GroupDetailSection groupDetailSection = formFactory.createGroupDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE| ExpandableComposite.EXPANDED);
        MemberDetailSection memberDetailSection = formFactory.createMemberDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        GrantedAuthorityDetailSection grantedAuthorityDetailSection = formFactory.createGrantedAuthorityDetailSection(getConversationHolder(), parent,ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(groupDetailSection);
        addPart(memberDetailSection);
        addPart(grantedAuthorityDetailSection);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jface.viewers.Viewer#getSelection()
     */
    /** {@inheritDoc} */
    @Override
    public ISelection getSelection() {
        return selection;
    }

    /** {@inheritDoc} */
    @Override
    public void setSelection(ISelection selection, boolean reveal) {
        this.selection = selection;
        if(this.selection!=null){
            SelectionChangedEvent selectionChangedEvent = new SelectionChangedEvent(this, selection);
            fireSelectionChanged(selectionChangedEvent);
        }
    }

    /**
     * createTaxonSections(RootElement parent)
     *
     * @param parent
     */
    private void createTaxonSections(RootElement parent) {
        destroySections();

        ParsingMessagesSection parsingMessagesSection = formFactory.createParsingMessagesSection(getConversationHolder(), parent, this,
                ExpandableComposite.EXPANDED | ExpandableComposite.EXPANDED);

        TaxonBaseDetailSection taxonBaseDetailSection = formFactory.createTaxonBaseDetailSection(getConversationHolder(), parent, this,
        		ExpandableComposite.TWISTIE	| ExpandableComposite.EXPANDED);
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NonViralNameDetailSection nonViralNameSection = formFactory.createNonViralNameDetailSection(getConversationHolder(), parent, this,
        		ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NomenclaturalReferenceDetailSection referenceDetailSection = formFactory.createNomenclaturalReferenceDetailSection(getConversationHolder(), parent, this,
        		ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NomenclaturalStatusSection nomenclaturalStatusSection = formFactory.createNomenclaturalStatusSection(getConversationHolder(), parent,
        		ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        //TODO RL
        if(!PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.IS_RL)){
	        ProtologueSection protologSection = formFactory.createProtologueSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);
	        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

	        TypeDesignationSection typeDesignationSection = formFactory.createTypeDesignationSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);
	        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

	        NameRelationshipDetailSection nameRelationshipSection = formFactory.createNameRelationshipDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);
	        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

			addPart(protologSection);
			addPart(typeDesignationSection);
			addPart(nameRelationshipSection);
        }


        addPart(parsingMessagesSection);
        addPart(taxonBaseDetailSection);
        addPart(nonViralNameSection);
        addPart(nomenclaturalStatusSection);
        addPart(referenceDetailSection);
    }

    /**
     * createNameSections
     *
     * @param parent
     */
    private void createNameSections(RootElement parent) {
        destroySections();
        NonViralNameDetailSection nonViralNameSection = formFactory.createNonViralNameDetailSection(getConversationHolder(), parent, this,
        		ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NomenclaturalReferenceDetailSection referenceDetailSection = formFactory.createNomenclaturalReferenceDetailSection(getConversationHolder(), parent, this,
        		ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NomenclaturalStatusSection nomenclaturalStatusSection = formFactory.createNomenclaturalStatusSection(getConversationHolder(), parent,
        		ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(nonViralNameSection);
        addPart(nomenclaturalStatusSection);
        addPart(referenceDetailSection);
//TODO RL
        if(!PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.IS_RL)){
			ProtologueSection protologSection = formFactory.createProtologueSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);
	        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

	        TypeDesignationSection typeDesignationSection = formFactory.createTypeDesignationSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);
	        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

	        NameRelationshipDetailSection nameRelationshipSection = formFactory.createNameRelationshipDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);
	        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
	        addPart(protologSection);
	        addPart(typeDesignationSection);
	        addPart(nameRelationshipSection);
        }
    }

    /**
     * createReferenceSections
     *
     * @param parent
     */
    private void createReferenceSections(RootElement parent) {
        destroySections();

        ReferenceDetailSection referenceDetailSection = formFactory.createReferenceDetailSection(getConversationHolder(), parent, this,
        		ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(referenceDetailSection);
    }

    /**
     * createTeamOrPersonBaseDetailSection
     *
     * @param parent
     */
    private void createTeamOrPersonBaseDetailSection(RootElement parent) {
        destroySections();
        TeamOrPersonBaseDetailSection teamOrPersonBaseDetailSection = formFactory.createTeamOrPersonBaseDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(teamOrPersonBaseDetailSection);
    }

    /**
     * createTeamDetailSection
     *
     * @param parent
     */
    private void createTeamDetailSection(RootElement parent) {
        destroySections();
        TeamDetailSection teamDetailSection = formFactory.createTeamDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(teamDetailSection);
    }

    /**
     * createPersonDetailSection
     *
     * @param parent
     */
    private void createPersonDetailSection(RootElement parent) {
        destroySections();
        PersonDetailSection personDetailSection = formFactory.createPersonDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(personDetailSection);
    }

    /**
     * createDescriptionElementSection
     *
     * @param parent
     */
    private void createDescriptionElementSection(RootElement parent) {
        destroySections();

        DescriptionElementDetailSection descriptionElementDetailSection = formFactory.createDescriptionElementDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DescriptionElementSourceSection descriptionElementSourceSection = formFactory.createDescriptionElementSourceSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DescriptionElementMediaSection descriptionElementMediaSection = formFactory.createDescriptionElementMediaSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(descriptionElementDetailSection);
        addPart(descriptionElementSourceSection);
        addPart(descriptionElementMediaSection);
    }

    /**
     * createDescriptionSection
     *
     * @param parent
     */
    private void createDescriptionSection(RootElement parent) {
        destroySections();
        DescriptionDetailSection descriptionDetailSection = formFactory.createDescriptionDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        NaturalLanguageSection naturalLanguageSection = formFactory.createNaturalLanguageSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DescribedSpecimenSection describedSpecimenSection = formFactory.createDescribedSpecimenSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        //		DescriptionSourceSection descriptionSourceSection = (DescriptionSourceSection) formFactory
        //				.createEntityDetailSection(EntityDetailType.DESCRIPTIONSOURCE,
        //						getConversationHolder(), parent, Section.TWISTIE);
        //
        //		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        ScopeSection scopeSection = formFactory.createScopeSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(descriptionDetailSection);
        addPart(naturalLanguageSection);
        addPart(describedSpecimenSection);
        //		addPart(descriptionSourceSection);
        addPart(scopeSection);
    }

    /**
     * createDescriptionSection
     *
     * @param parent
     */
    private void createSpecimenDescriptionSection(RootElement parent) {
        destroySections();
        DescriptionDetailSection descriptionDetailSection = formFactory.createDescriptionDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(descriptionDetailSection);
    }

    /**
     * Creates the use Description section
     *
     * @param parent
     */
    private void createUseDescriptionSection(RootElement parent) {
        destroySections();
        DescriptionDetailSection descriptionDetailSection = formFactory.createDescriptionDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DescriptionSourceSection descriptionSourceSection = formFactory.createDescriptionSourceSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(descriptionDetailSection);
        addPart(descriptionSourceSection);

    }


    /**
     * @param rootElement
     */
    private void createImageGallerySection(RootElement parent) {
        destroySections();
        DescriptionDetailSection descriptionDetailSection = formFactory.createDescriptionDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(descriptionDetailSection);
    }

    /**
     * createMediaElementSection
     *
     * @param parent
     */

    private void createMediaElementSection(RootElement parent) {
        destroySections();

        MediaDetailsSection mediaDetailSection = formFactory.createMediaDetailsSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(mediaDetailSection);
    }

    /**
     * createDerivedUnitBaseElementSection
     *
     * @param parent
     */
    private void createDerivedUnitBaseElementSection(RootElement parent) {
        destroySections();

        DerivedUnitGeneralDetailSection derivedUnitGeneralDetailSection = formFactory.createDerivedUnitGeneralDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        //TODO for DerivateEditor do not use facade anymore to avoid this special case handling #4539
        if(!(AbstractUtility.getActivePart() instanceof IDerivedUnitFacadePart)){
            derivedUnitGeneralDetailSection.setShowOnlyDerivedUnitData(true);
        }
        addPart(derivedUnitGeneralDetailSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DerivedUnitBaseDetailSection derivedUnitBaseDetailSection = formFactory.createDerivedUnitBaseDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(derivedUnitBaseDetailSection);

        //for editors working with facades
        if(AbstractUtility.getActivePart() instanceof IDerivedUnitFacadePart){
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            GatheringEventDetailSection gatheringEventDetailSection = formFactory.createGatheringEventDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE);
            addPart(gatheringEventDetailSection);

            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            FieldUnitDetailSection fieldUnitDetailSection = formFactory.createFieldUnitDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE);
            addPart(fieldUnitDetailSection);
        }

        if(!PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.DETERMINATION_ONLY_FOR_FIELD_UNITS)){
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            DeterminationDetailSection determinationDetailSection = formFactory.createDeterminationDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE);
            addPart(determinationDetailSection);
        }
        if(PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.SHOW_TAXON_ASSOCIATIONS)){
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            TaxonAssociationDetailSection taxonAssociationDetailSection = formFactory.createTaxonAssociationDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
            addPart(taxonAssociationDetailSection);
        }

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        IdentifierDetailSection identifierDetailSection = formFactory.createIdentifierDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);
        addPart(identifierDetailSection);

    }

    private void createFieldUnitSection(RootElement parent) {
        destroySections();

        FieldUnitGeneralDetailSection fielUnitGeneralDetailSection = formFactory.createFieldUnitGeneralDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        GatheringEventDetailSection gatheringEventDetailSection = formFactory.createGatheringEventDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        FieldUnitDetailSection fieldUnitDetailSection = formFactory.createFieldUnitDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE);

        addPart(fielUnitGeneralDetailSection);
        addPart(gatheringEventDetailSection);
        addPart(fieldUnitDetailSection);

        if(PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.DETERMINATION_ONLY_FOR_FIELD_UNITS)){
            formFactory.createHorizontalSeparator(parent, SWT.BORDER);
            DeterminationDetailSection determinationDetailSection = formFactory.createDeterminationDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE);
            addPart(determinationDetailSection);
        }
    }

    private void createTissueSampleSection(RootElement parent) {
        destroySections();

        TissueSampleGeneralDetailSection section = formFactory.createTissueSampleGeneralDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SampleDesignationDetailSection sampleDesignationDetailSection = formFactory.createSampleDesignationDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(section);
        addPart(sampleDesignationDetailSection);
    }

    private void createDnaSampleSection(RootElement parent) {
        destroySections();

        DnaSampleGeneralDetailSection section = formFactory.createDnaSampleGeneralDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DnaSamplePreparationPreservationSection preparationPreservationSection = formFactory.createDnaSamplePreparationPreservationSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        DnaQualityDetailSection qualitySection = formFactory.createDnaQualityDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SampleDesignationDetailSection sampleDesignationDetailSection = formFactory.createSampleDesignationDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(section);
        addPart(preparationPreservationSection);
        addPart(qualitySection);
        addPart(sampleDesignationDetailSection);
    }

    private void createSequenceSection(RootElement parent) {
        destroySections();

        SequenceGeneralDetailSection section = formFactory.createSequenceGeneralDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SequenceReferenceCollectionDetailSection referenceSection = formFactory.createSequenceReferenceCollectionDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SequenceContigFileCollectionDetailSection contigFileSection = formFactory.createSequenceContigFileCollectionDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);


        addPart(section);
        addPart(referenceSection);
        addPart(contigFileSection);
    }

    private void createSingleReadSection(RootElement parent) {
        destroySections();

        SingleReadGeneralDetailSection section = formFactory.createSingleReadGeneralDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(section);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        SingleReadPherogramCollectionDetailSection pherogramSection = formFactory.createSingleReadPherogramCollectionDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(pherogramSection);

    }

    private void createMediaSpecimenSection(RootElement parent) {
        destroySections();

        MediaSpecimenGeneralDetailSection generalSection = formFactory.createMediaSpecimenGeneralDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(generalSection);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        RightsSection rightsSection = formFactory.createRightsSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE);
        addPart(rightsSection);

    }



    /**
     * @param rootElement
     */
    private void createFeatureDistributionSection(RootElement parent) {
        destroySections();

        FeatureDistributionDetailSection featureDistributionSection = formFactory.createFeatureDistributionDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(featureDistributionSection);
    }

    /**
     * createPolytomousKeyNodeSection
     *
     * @param parent
     */
    private void createPolytomousKeyNodeSection(RootElement parent) {
        destroySections();

        PolytomousKeyDetailSection polytomousKeyDetailSection = formFactory.createPolytomousKeyDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.COMPACT);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        PolytomousKeyNodeDetailSection polytomousKeyNodeDetailSection = formFactory.createPolytomousKeyNodeDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        TaxonomicScopeSection taxonomicScopeSection = formFactory.createTaxonomicScopeSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.COMPACT);

        addPart(polytomousKeyDetailSection);
        addPart(polytomousKeyNodeDetailSection);
        addPart(taxonomicScopeSection);

    }

    /**
     * create the UseRecordSection
     *
     * @param parent
     */
    private void createUseRecordSection(RootElement parent) {
        destroySections();

        UseRecordDetailSection descriptionUseRecordSection = formFactory.createUseRecordDetailSection(getConversationHolder(), parent, this,ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        addPart(descriptionUseRecordSection);


    }

    /**
     * createUserSection
     *
     * @param parent
     */
    private void createUserSection(RootElement parent) {
        destroySections();

        UserDetailSection userDetailSection = formFactory.createUserDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        GroupsByUserDetailSection groupByUserDetailSection = formFactory.createGroupsByUserDetailSection(getConversationHolder(), parent, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        addPart(userDetailSection);
        addPart(groupByUserDetailSection);
    }

    /**
     * createTaxonRelationshipSection
     *
     * @param parent
     */
    private void createTaxonRelationshipSection(RootElement parent) {
        destroySections();

        TaxonRelationshipDetailSection taxonRelationshipDetailSection = formFactory.createTaxonRelationshipDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);

        ReferencedEntityDetailSection referencedEntityBaseDetailSection = formFactory.createReferencedEntityDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(taxonRelationshipDetailSection);
        addPart(referencedEntityBaseDetailSection);
    }

    /**
     * @param rootElement
     */
    private void createTermVocabularySection(RootElement parent) {
        destroySections();

        TermVocabularyDetailSection termVocabularyDetailSection = formFactory.createTermVocabularyDetailSection(getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(termVocabularyDetailSection);
    }

    /**
     * createDefinedTermSection
     *
     * @param parent
     */
    private void createDefinedTermSection(RootElement parent) {
        destroySections();

        AbstractFormSection<?> definedTermDetailSection = formFactory.createDefinedTermDetailSection(getInput().getClass(), getConversationHolder(), parent, this, ExpandableComposite.TWISTIE | ExpandableComposite.EXPANDED);
        addPart(definedTermDetailSection);
    }

}
