// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import java.net.URI;
import java.net.URISyntaxException;

import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.common.DefinedTerm;
import eu.etaxonomy.cdm.model.common.TermType;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SequenceString;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author pplitzner
 * @date 19.12.2013
 *
 */
public class SequenceGeneralDetailElement extends AbstractCdmDetailElement<Sequence> {

    private TermComboElement<DefinedTerm> comboMarker;
    private TextWithLabelElement textConsensusSequence;
    private TextWithLabelElement textGeneticAccessNo;
    private UriWithLabelElement textNCBIUri;
    private UriWithLabelElement textENAUri;
    private UriWithLabelElement textDDBJUri;
    private TextWithLabelElement textBoldProcessID;
    private UriWithLabelElement textBoldUri;

    /**
     * @param formFactory
     * @param formElement
     */
    public SequenceGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
     * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, java.lang.Object, int)
     */
    @Override
    protected void createControls(ICdmFormElement formElement, Sequence entity, int style) {
        comboMarker = formFactory.createDefinedTermComboElement(TermType.DnaMarker, formElement, "DNA Marker", entity.getDnaMarker(), style);
        textConsensusSequence = formFactory.createTextWithLabelElement(formElement, "Consensus Sequence 5'->3'", entity.getConsensusSequence().getString(), style);

        URI genBankUri = null;
        URI emblUri = null;
        URI ddbjUri = null;
        URI boldUri = null;
        try{
            genBankUri = entity.getGenBankUri();
            emblUri = entity.getEmblUri();
            ddbjUri = entity.getDdbjUri();
            boldUri = entity.getBoldUri();
        }
        catch(URISyntaxException e){

        }
        textGeneticAccessNo = formFactory.createTextWithLabelElement(formElement, "Genetic Acc. No.", entity.getGeneticAccessionNumber(), style);
        textNCBIUri = formFactory.createUriWithLabelElement(formElement, "NCBI URI", genBankUri, style);
        ((Text) textNCBIUri.getMainControl()).setEditable(false);
        textENAUri = formFactory.createUriWithLabelElement(formElement, "ENA URI", emblUri, style);
        ((Text) textENAUri.getMainControl()).setEditable(false);
        textDDBJUri = formFactory.createUriWithLabelElement(formElement, "DDBJ URI", ddbjUri, style);
        ((Text) textDDBJUri.getMainControl()).setEditable(false);
        textBoldProcessID = formFactory.createTextWithLabelElement(formElement, "BOLD Process ID", entity.getBoldProcessId(), style);
        textBoldUri = formFactory.createUriWithLabelElement(formElement, "BOLD URI", boldUri, style);
        ((Text) textBoldUri.getMainControl()).setEditable(false);
        ((Text) textBoldUri.getMainControl()).setToolTipText("Query BOLD");
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
     * .lang.Object)
     */
    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==comboMarker){
            getEntity().setDnaMarker(comboMarker.getSelection());
        }
        else if(eventSource==textConsensusSequence){
            SequenceString consensusSequence = getEntity().getConsensusSequence();
            if(consensusSequence==null){
                consensusSequence = SequenceString.NewInstance();
            }
            consensusSequence.setString(textConsensusSequence.getText());
            getEntity().setConsensusSequence(consensusSequence);
        }
        else if(eventSource==textGeneticAccessNo){
            textGeneticAccessNo.setBackground(getPersistentBackground());
            getEntity().setGeneticAccessionNumber(textGeneticAccessNo.getText());
            if(textGeneticAccessNo.getText()!=null && !textGeneticAccessNo.getText().isEmpty()){
                try {
                    textNCBIUri.setUri(getEntity().getGenBankUri());
                    textENAUri.setUri(getEntity().getEmblUri());
                    textDDBJUri.setUri(getEntity().getDdbjUri());
                } catch (URISyntaxException e) {
                    textGeneticAccessNo.setBackground(getColor(Resources.COLOR_PARSE_ERROR));
                }
            }
            else{
                textNCBIUri.setText("");
                textENAUri.setText("");
                textDDBJUri.setText("");
            }
        }
        else if(eventSource==textBoldProcessID){
            textBoldProcessID.setBackground(getPersistentBackground());
            getEntity().setBoldProcessId(textBoldProcessID.getText());
            if(textBoldProcessID.getText()!=null && !textBoldProcessID.getText().isEmpty()){
                try {
                    textBoldUri.setUri(getEntity().getBoldUri());
                } catch (URISyntaxException e) {
                    textBoldProcessID.setBackground(getColor(Resources.COLOR_PARSE_ERROR));
                }
            }
            else{
                textBoldUri.setText("");
            }
        }
    }

}
